'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import config
from taskcoachlib.domain import task, effort, date
from taskcoachlib.thirdparty.pubsub import pub
import test


class FakeEffortAggregator(object):
    def __init__(self, composite):
        self.composite = composite
        pub.subscribe(self.onTimeSpentChanged,
                      task.Task.timeSpentChangedEventType())
        pub.subscribe(self.onRevenueChanged,
                      task.Task.hourlyFeeChangedEventType())
        
    def onTimeSpentChanged(self, newValue, sender):
        self.composite.onTimeSpentChanged(newValue, sender)
            
    def onTrackingChanged(self, newValue, sender):
        self.composite.onTrackingChanged(newValue, sender)
            
    def onRevenueChanged(self, newValue, sender):
        self.composite.onRevenueChanged(newValue, sender)


class CompositeEffortWithRoundingTest(test.TestCase):
    def setUp(self):
        task.Task.settings = config.Settings(load=False)
        self.task = task.Task(subject='task')
        self.effort1 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 11, 0, 45))
        self.effort2 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 12, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 45))
        self.effort3 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 13, 0, 0), 
            date.DateTime(2004, 1, 1, 13, 0, 45))
        self.effort4 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 13, 0, 0), 
            date.DateTime(2004, 1, 1, 13, 0, 10))
        self.composite = effort.CompositeEffort(self.task,
            date.DateTime(2004, 1, 1, 0, 0, 0),
            date.DateTime(2004, 1, 1, 23, 59, 59))
        self.task.addEffort(self.effort1)
        self.task.addEffort(self.effort2)
        self.task.addEffort(self.effort3)
        self.task.addEffort(self.effort4)

    def test_round_total(self):
        self.assertEqual(self.composite.duration(recursive=True, rounding=60), date.TimeDelta(seconds=3*60))

    def test_round_total_up(self):
        self.assertEqual(self.composite.duration(recursive=True, rounding=60, roundUp=True), date.TimeDelta(seconds=4*60))


class CompositeEffortTest(test.TestCase):
    def setUp(self):
        task.Task.settings = config.Settings(load=False)
        self.task = task.Task(subject='task')
        self.effort1 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 0))
        self.effort2 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 13, 0, 0), 
            date.DateTime(2004, 1, 1, 14, 0, 0))
        self.effort3 = effort.Effort(self.task, 
            date.DateTime(2004, 1, 11, 13, 0, 0), 
            date.DateTime(2004, 1, 11, 14, 0, 0))
        self.trackedEffort = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 9, 0, 0))
        self.composite = effort.CompositeEffort(self.task,
            date.DateTime(2004, 1, 1, 0, 0, 0),
            date.DateTime(2004, 1, 1, 23, 59, 59))
        self.fakeAggregator = FakeEffortAggregator(self.composite)
        
    def testInitialLength(self):
        self.assertEqual(0, len(self.composite))

    def testInitialDuration(self):
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testInitialTrackingState(self):
        self.failIf(self.composite.isBeingTracked())

    def testInitialTrackingStateWhenTaskIsTracked(self):
        self.task.addEffort(self.trackedEffort)
        composite = effort.CompositeEffort(self.task, 
            self.composite.getStart(), self.composite.getStop())
        self.failUnless(composite.isBeingTracked())
        
    def testDurationForSingleEffort(self):
        self.task.addEffort(self.effort1)
        self.assertEqual(self.effort1.duration(), self.composite.duration())

    def testAddEffortOutsidePeriodToTask(self):
        effortOutsidePeriod = effort.Effort(self.task, 
            date.DateTime(2004, 1, 11, 13, 0, 0), 
            date.DateTime(2004, 1, 11, 14, 0, 0))
        self.task.addEffort(effortOutsidePeriod)
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testAddEffortWithStartTimeEqualToStartOfPeriodToTask(self):
        effortSameStartTime = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 0, 0, 0), 
            date.DateTime(2004, 1, 1, 14, 0, 0))
        self.task.addEffort(effortSameStartTime)
        self.assertEqual(effortSameStartTime.duration(), 
            self.composite.duration())

    def testAddEffortWithStartTimeEqualToEndOfPeriodToTask(self):
        effortSameStopTime = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 23, 59, 59), 
            date.DateTime(2004, 1, 2, 1, 0, 0))
        self.task.addEffort(effortSameStopTime)
        self.assertEqual(effortSameStopTime.duration(), 
            self.composite.duration())

    def testAddTrackedEffortToTaskDoesNotCauseListEmptyNotification(self):
        events = []
        
        def onEvent(sender):
            events.append(sender)
            
        pub.subscribe(onEvent, effort.CompositeEffort.compositeEmptyEventType())
        self.task.addEffort(effort.Effort(self.task, self.composite.getStart()))
        self.failIf(events)

    def testAddEffortNotification(self):
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.durationChangedEventType())
        self.task.addEffort(self.effort1)
        self.assertEqual([(self.composite.duration(), self.composite)], events)

    def testRemoveEffortFromTask(self):
        self.task.addEffort(self.effort1)
        self.task.removeEffort(self.effort1)
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testRemoveEffortNotification(self):
        self.task.addEffort(self.effort1)
        events = []
        
        def onEvent(sender):
            events.append(sender)
            
        pub.subscribe(onEvent, effort.CompositeEffort.compositeEmptyEventType())
        self.task.removeEffort(self.effort1)
        self.assertEqual([self.composite], events)

    def testDuration(self):
        self.task.addEffort(self.effort1)
        self.assertEqual(self.effort1.duration(), self.composite.duration())

    def testDurationTwoEfforts(self):
        self.task.addEffort(self.effort1)
        self.task.addEffort(self.effort2)
        self.assertEqual(self.effort1.duration() + self.effort2.duration(), 
            self.composite.duration())

    def testRevenue(self):
        self.task.setHourlyFee(100)
        self.task.addEffort(self.effort1)
        self.assertEqual(100, self.composite.revenue())

    def testRevenueTwoEfforts(self):
        self.task.setHourlyFee(100)
        self.task.addEffort(self.effort1)
        self.task.addEffort(self.effort2)
        self.assertEqual(200, self.composite.revenue())

    def testThatAnHourlyFeeChangeCausesARevenueNotification(self):
        self.task.addEffort(self.effort1)
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.revenueChangedEventType())
        self.task.setHourlyFee(100)
        self.failUnless((100.0, self.composite) in events)

    def testIsBeingTracked(self):
        self.task.addEffort(self.effort1)
        self.effort1.setStop(date.DateTime())
        self.failUnless(self.composite.isBeingTracked())

    def testChangeStartTimeOfEffort_KeepWithinPeriod(self):
        self.task.addEffort(self.effort1)
        self.effort1.setStart(self.effort1.getStart() + date.ONE_HOUR)
        self.assertEqual(self.effort1.duration(), self.composite.duration())

    def testChangeStartTimeOfEffort_KeepWithinPeriod_NoNotification(self):
        self.task.addEffort(self.effort1)
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.durationChangedEventType())
        self.effort1.setStart(self.effort1.getStart() + date.ONE_HOUR)
        self.failIf((self.composite.duration(), self.composite) in events)

    def testChangeStartTimeOfEffort_MoveOutsidePeriode(self):
        self.task.addEffort(self.effort1)
        self.effort1.setStart(self.effort1.getStart() + date.TimeDelta(days=2))
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testChangeStopTimeOfEffort_KeepWithinPeriod(self):
        self.task.addEffort(self.effort1)
        self.effort1.setStop(self.effort1.getStop() + date.ONE_HOUR)
        self.assertEqual(self.effort1.duration(), self.composite.duration())

    def testChangeStopTimeOfEffort_MoveOutsidePeriod(self):
        self.task.addEffort(self.effort1)
        self.effort1.setStop(self.effort1.getStop() + date.TimeDelta(days=2))
        self.assertEqual(self.effort1.duration(), self.composite.duration())

    def testChangeStopTimeOfEffort_MoveOutsidePeriod_Notification(self):
        self.task.addEffort(self.effort1)
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.durationChangedEventType())
        self.effort1.setStop(self.effort1.getStop() + date.TimeDelta(days=2))
        self.failIf((self.composite.duration(), self.composite) in events)
        
    def testChangeStopTimeOfEffort_NoNotification(self):
        self.task.addEffort(self.effort1)
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.durationChangedEventType())
        self.effort1.setStop(self.effort1.getStop() + date.ONE_HOUR)
        self.failIf((self.composite.duration(), self.composite) in events)

    def testChangeStartTimeOfEffort_MoveInsidePeriod(self):
        self.task.addEffort(self.effort3)
        self.effort3.setStart(self.composite.getStart())
        self.assertEqual(self.effort3.duration(), self.composite.duration())

    def testEmptyNotification(self):
        events = []
        
        def onEvent(sender):
            events.append(sender)
            
        pub.subscribe(onEvent, effort.CompositeEffort.compositeEmptyEventType())
        self.task.addEffort(self.effort1)
        self.task.removeEffort(self.effort1)
        self.assertEqual([self.composite], events)
                
    def testChangeTask(self):
        self.task.addEffort(self.effort1)
        self.effort1.setTask(task.Task())
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testChangeTask_EmptyNotification(self):
        events = []
        
        def onEvent(sender):
            events.append(sender)
            
        pub.subscribe(onEvent, effort.CompositeEffort.compositeEmptyEventType())
        self.task.addEffort(self.effort1)
        self.effort1.setTask(task.Task())
        self.assertEqual([self.composite], events)

    def testGetDescription_ZeroEfforts(self):
        self.assertEqual('', self.composite.description())
        
    def testGetDescription_OneEffort(self):
        self.task.addEffort(self.effort1)
        for description in ('', 'Description'):
            self.effort1.setDescription(description)
            self.assertEqual(description, self.composite.description())
        
    def testGetDescription_TwoEfforts(self):
        self.task.addEffort(self.effort1)
        self.task.addEffort(self.effort2)
        for description1, description2 in (('', ''), ('descripion1', ''), 
                                           ('', 'description2'), 
                                           ('description1', 'description2')):
            self.effort1.setDescription(description1)
            self.effort2.setDescription(description2)
            if description1 and description2:
                seperator = '\n'
            else:
                seperator = ''
            expectedDescription = description1 + seperator + description2
            self.assertEqual(expectedDescription, self.composite.description())


class CompositeEffortWithSubTasksTest(test.TestCase):
    def setUp(self):
        task.Task.settings = config.Settings(load=False)
        self.task = task.Task(subject='task')
        self.child = task.Task(subject='child')
        self.child2 = task.Task(subject='child2')
        self.task.addChild(self.child)
        self.taskEffort = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 0))
        self.childEffort = effort.Effort(self.child, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 0))
        self.child2Effort = effort.Effort(self.child2, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 0))
        self.trackedEffort = effort.Effort(self.child, 
            date.DateTime(2004, 1, 1, 9, 0, 0))
        self.composite = effort.CompositeEffort(self.task,
            date.DateTime(2004, 1, 1, 0, 0, 0), 
            date.DateTime(2004, 1, 1, 23, 59, 59))
        self.fakeAggregator = FakeEffortAggregator(self.composite)

    def testAddEffortToChildTaskNotification(self):
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.durationChangedEventType())
        self.child.addEffort(self.childEffort)
        self.failUnless((self.composite.duration(), self.composite) in events)

    def testRemoveEffortFromChildTask(self):
        self.child.addEffort(self.childEffort)
        self.child.removeEffort(self.childEffort)
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testRemoveEffortFromChildNotification(self):
        self.child.addEffort(self.childEffort)
        events = []
        
        def onEvent(sender):
            events.append(sender)
            
        pub.subscribe(onEvent, effort.CompositeEffort.compositeEmptyEventType())
        self.child.removeEffort(self.childEffort)
        self.assertEqual([self.composite], events)

    def testDuration(self):
        self.child.addEffort(self.childEffort)
        self.assertEqual(date.TimeDelta(), self.composite.duration())

    def testRecursiveDuration(self):
        self.child.addEffort(self.childEffort)
        self.assertEqual(self.childEffort.duration(), 
            self.composite.duration(recursive=True))

    def testDurationWithTaskAndChildEffort(self):
        self.task.addEffort(self.taskEffort)
        self.child.addEffort(self.childEffort)
        self.assertEqual(self.taskEffort.duration() + \
            self.childEffort.duration(), 
            self.composite.duration(recursive=True))

    def testAddEffortToNewChild(self):
        self.task.addChild(self.child2)
        self.child2.addEffort(self.child2Effort)
        self.assertEqual(self.child2Effort.duration(), 
            self.composite.duration(recursive=True))

    def testAddChildWithEffort(self):
        self.child2.addEffort(self.child2Effort)
        self.task.addChild(self.child2)
        self.assertEqual(self.child2Effort.duration(), 
            self.composite.duration(recursive=True))

    def testAddEffortToGrandChild(self):
        self.task.addChild(self.child2)
        grandChild = task.Task(subject='grandchild')
        self.child2.addChild(grandChild)
        grandChildEffort = effort.Effort(grandChild, self.composite.getStart(), 
            self.composite.getStart() + date.ONE_HOUR)
        grandChild.addEffort(grandChildEffort)
        self.assertEqual(grandChildEffort.duration(),
            self.composite.duration(recursive=True))

    def testAddGrandChildWithEffort(self):
        self.task.addChild(self.child2)
        grandChild = task.Task(subject='grandchild')
        grandChildEffort = effort.Effort(grandChild, self.composite.getStart(),
            self.composite.getStart() + date.ONE_HOUR)
        grandChild.addEffort(grandChildEffort)
        self.child2.addChild(grandChild)
        self.assertEqual(grandChildEffort.duration(),
            self.composite.duration(recursive=True))

    def testRemoveEffortFromAddedChild(self):
        self.task.addChild(self.child2)
        self.child2.addEffort(self.child2Effort)
        self.child2.removeEffort(self.child2Effort)
        self.assertEqual(date.TimeDelta(), 
            self.composite.duration(recursive=True))

    def testRemoveChildWithEffort(self):
        self.child.addEffort(self.childEffort)
        self.task.removeChild(self.child)
        self.assertEqual(date.TimeDelta(), 
            self.composite.duration(recursive=True))

    def testRemoveChildWithEffortCausesEmptyNotification(self):
        events = []
        
        def onEvent(sender):
            events.append(sender)
            
        pub.subscribe(onEvent, effort.CompositeEffort.compositeEmptyEventType())
        self.child.addEffort(self.childEffort)
        self.task.removeChild(self.child)
        self.assertEqual([self.composite], events)

    def testChangeStartTimeOfChildEffort_MoveInsidePeriod(self):
        childEffort = effort.Effort(self.child)
        self.child.addEffort(childEffort)
        childEffort.setStart(self.composite.getStart())
        # Make sure the next assertEqual cannot fail due to duration() being 
        # called twice:
        childEffort.setStop() 
        self.assertEqual(childEffort.duration(),
            self.composite.duration(recursive=True))

    def testChangeTask(self):
        self.child.addEffort(self.childEffort)
        self.childEffort.setTask(task.Task())
        self.assertEqual(date.TimeDelta(), 
            self.composite.duration(recursive=True))


class CompositeEffortWithSubTasksRevenueTest(test.TestCase):
    def setUp(self):
        task.Task.settings = config.Settings(load=False)
        self.task = task.Task(subject='task')
        self.child = task.Task(subject='child')
        self.task.addChild(self.child)
        self.taskEffort = effort.Effort(self.task, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 0))
        self.childEffort = effort.Effort(self.child, 
            date.DateTime(2004, 1, 1, 11, 0, 0), 
            date.DateTime(2004, 1, 1, 12, 0, 0))
        self.composite = effort.CompositeEffort(self.task,
            date.DateTime(2004, 1, 1, 0, 0, 0), 
            date.DateTime(2004, 1, 1, 23, 59, 59))
        self.fakeAggregator = FakeEffortAggregator(self.composite)
        self.task.addEffort(self.taskEffort)
        self.child.addEffort(self.childEffort)
                
    def testRevenueWhenParentHasHourlyFee(self):
        self.task.setHourlyFee(100)
        self.assertEqual(self.taskEffort.duration().hours() * 100,
            self.composite.revenue())

    def testRecursiveRevenueWhenParentHasHourlyFee(self):
        self.task.setHourlyFee(100)
        self.assertEqual(self.taskEffort.duration().hours() * 100,
            self.composite.revenue(recursive=True))

    def testRevenueWhenChildHasHourlyFee(self):
        self.child.setHourlyFee(100)
        self.assertEqual(0, self.composite.revenue())

    def testRecursiveRevenueWhenChildHasHourlyFee(self):
        self.child.setHourlyFee(100)
        self.assertEqual(self.childEffort.duration().hours() * 100, 
            self.composite.revenue(recursive=True))

    def testRevenueWhenChildAndParentHaveHourlyFees(self):
        self.child.setHourlyFee(100)
        self.task.setHourlyFee(200)
        self.assertEqual(self.taskEffort.duration().hours() * 200, 
            self.composite.revenue())

    def testRecursiveRevenueWhenChildAndParentHaveHourlyFees(self):
        self.child.setHourlyFee(100)
        self.task.setHourlyFee(200)
        self.assertEqual(self.taskEffort.duration().hours() * 200 + \
            self.childEffort.duration().hours() * 100, 
            self.composite.revenue(recursive=True))

    def testRevenueWhenParentHasFixedFee(self):
        self.task.setFixedFee(1000)
        self.assertEqual(0, self.composite.revenue())

    def testRecursiveRevenueWhenParentHasFixedFee(self):
        self.task.setFixedFee(1000)
        self.assertEqual(0, self.composite.revenue(recursive=True))

    def testRevenueWhenChildHasFixedFee(self):
        self.child.setFixedFee(1000)
        self.assertEqual(0, self.composite.revenue())

    def testRecursiveRevenueWhenChildHasFixedFee(self):
        self.child.setFixedFee(1000)
        self.assertEqual(0, self.composite.revenue(recursive=True))

    def testRevenueWhenParentHasFixedFeeAndMultipleEfforts(self):
        self.task.setFixedFee(1000)
        self.task.addEffort(effort.Effort(self.task, 
            date.DateTime(2005, 12, 12, 10, 0, 0), 
            date.DateTime(2005, 12, 12, 12, 0, 0)))
        self.assertEqual(0, self.composite.revenue())

    def testRevenueWhenChildHasFixedFeeAndMultipleEfforts(self):
        self.child.setFixedFee(1000)
        self.child.addEffort(effort.Effort(self.child, 
            date.DateTime(2005, 12, 12, 10, 0, 0), 
            date.DateTime(2005, 12, 12, 12, 0, 0)))
        self.assertEqual(0, self.composite.revenue())

    def testRecursiveRevenueWhenChildHasFixedFeeAndMultipleEfforts(self):
        self.child.setFixedFee(1000)
        self.child.addEffort(effort.Effort(self.child, 
            date.DateTime(2005, 12, 12, 10, 0, 0), 
            date.DateTime(2005, 12, 12, 12, 0, 0)))
        self.assertEqual(0, self.composite.revenue(recursive=True))

    def testRevenueWithMixture(self):
        self.child.setFixedFee(100)
        self.task.setHourlyFee(1000)
        self.assertEqual(1000, self.composite.revenue(recursive=True))

    def testThatAnHourlyFeeChangeCausesARevenueNotification(self):
        events = []
        
        def onEvent(newValue, sender):
            events.append((newValue, sender))
            
        pub.subscribe(onEvent, effort.Effort.revenueChangedEventType())
        self.child.setHourlyFee(100)
        self.failUnless((0.0, self.composite) in events)



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test
from taskcoachlib import persistence
from taskcoachlib.domain import date 


class VCalendarParserTest(test.TestCase):
    # pylint: disable=W0511

    def setUp(self):
        self.parser = persistence.icalendar.ical.VCalendarParser()
        
    def testEmptyVCalender(self):
        self.parser.parse(['BEGIN:VCALENDAR', 'END:VCALENDAR'])
        self.failIf(self.parser.tasks)
           
    def testEmptyVTodo(self):
        self.parser.parse(['BEGIN:VTODO', 'END:VTODO'])
        self.assertEqual(dict(status=0, plannedStartDateTime=date.DateTime()), 
                         self.parser.tasks[0])
        
    def testSubject(self):
        self.parser.parse(['BEGIN:VTODO', 'SUBJECT:Test', 'END:VTODO'])
        self.assertEqual(dict(status=0, subject='Test', 
                              plannedStartDateTime=date.DateTime()), 
                         self.parser.tasks[0])
        
    def testDueDate(self):
        self.parser.parse(['BEGIN:VTODO', 'DUE:20100101T120000', 'END:VTODO'])
        self.assertEqual(dict(status=0, 
                              dueDateTime=date.DateTime(2010, 1, 1, 12, 0, 0), 
                              plannedStartDateTime=date.DateTime()), 
                         self.parser.tasks[0])

    def testPercentageComplete(self):
        self.parser.parse(['BEGIN:VTODO', 'PERCENT-COMPLETE:56', 'END:VTODO'])
        self.assertEqual(dict(status=0, percentageComplete=56, 
                              plannedStartDateTime=date.DateTime()), 
                         self.parser.tasks[0])
        
    def testCreationDateTime(self):
        self.parser.parse(['BEGIN:VTODO', 'CREATED:20100101T120000', 
                           'END:VTODO'])
        self.assertEqual(dict(status=0,
                              creationDateTime=date.DateTime(2010, 1, 1, 12, 0, 0),
                              plannedStartDateTime=date.DateTime()),
                         self.parser.tasks[0])
        
    def testModificationDateTime(self):
        self.parser.parse(['BEGIN:VTODO', 'LAST-MODIFIED:20100101T120000', 
                           'END:VTODO'])
        self.assertEqual(dict(status=0,
                              modificationDateTime=date.DateTime(2010, 1, 1, 12, 0, 0),
                              plannedStartDateTime=date.DateTime()),
                         self.parser.tasks[0])


class VNoteParserTest(test.TestCase):
    def setUp(self):
        self.parser = persistence.icalendar.ical.VNoteParser()
        
    def testEmptyVCalendar(self):
        self.parser.parse(['BEGIN:VCALENDAR', 'END:VCALENDAR'])
        self.failIf(self.parser.notes)
    
    def testEmptyVNote(self):
        self.parser.parse(['BEGIN:VNOTE', 'END:VNOTE'])
        self.assertEqual(dict(status=0, subject=''), self.parser.notes[0])

    def testSubject(self):
        self.parser.parse(['BEGIN:VNOTE', 'SUMMARY:Subject', 'END:VNOTE'])
        self.assertEqual(dict(status=0, subject='Subject'), 
                         self.parser.notes[0])
    
    def testCreationDateTime(self):
        self.parser.parse(['BEGIN:VNOTE', 'CREATED:20100101T120000', 
                           'END:VNOTE'])
        self.assertEqual(dict(status=0, subject='', 
                              creationDateTime=date.DateTime(2010, 1, 1, 12, 0, 0)), 
                         self.parser.notes[0])
    
    def testModificationDateTime(self):
        self.parser.parse(['BEGIN:VNOTE', 'LAST-MODIFIED:20100101T120000', 
                           'END:VNOTE'])
        self.assertEqual(dict(status=0, subject='', 
                              modificationDateTime=date.DateTime(2010, 1, 1, 12, 0, 0)), 
                         self.parser.notes[0])



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test, wx
from taskcoachlib.widgets import treectrl


class DummyEvent(object):
    def __init__(self, item=None):
        self.item = item
        self.vetoed = self.allowed = False
        
    def GetItem(self):
        return self.item
    
    def Veto(self):
        self.vetoed = True
        
    def Allow(self):
        self.allowed = True
    
    
class TreeCtrlDragAndDropMixinTest(test.wxTestCase):
    # pylint: disable=E1101
    
    def setUp(self):
        self.treeCtrl = treectrl.HyperTreeList(self.frame)
        self.treeCtrl.AddColumn('First')
        
        self.rootItem = self.treeCtrl.AddRoot('root')
        self.item = self.treeCtrl.AppendItem(self.rootItem, 'item')
        
    def assertEventIsVetoed(self, event):
        self.failUnless(event.vetoed)
        self.failIf(event.allowed)
        
    def assertEventIsAllowed(self, event):
        self.failUnless(event.allowed)
        self.failIf(event.vetoed)
        
    def testEventIsVetoedWhenDragBeginsWithoutItem(self): 
        event = DummyEvent()
        self.treeCtrl._dragStartPos = wx.Point(0, 0)
        self.treeCtrl.OnBeginDrag(event)
        self.assertEventIsVetoed(event)
        
    def testEventIsAllowedWhenDragBeginsWithItem(self):
        event = DummyEvent(self.item)
        self.treeCtrl._dragStartPos = wx.Point(0, 0)
        self.treeCtrl.OnBeginDrag(event)
        self.assertEventIsAllowed(event)
        
    def testEventIsAllowedWhenDragBeginWithSelectedItem(self):
        self.treeCtrl.SelectItem(self.item)
        event = DummyEvent(self.item)
        self.treeCtrl._dragStartPos = wx.Point(0, 0)
        self.treeCtrl.OnBeginDrag(event)
        self.assertEventIsAllowed(event)



'''
Task Coach - Your friendly task manager
Copyright (C) 2011-2012 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
import wx.lib.colourselect as csel
from wx.lib import sized_controls
from taskcoachlib.i18n import _
from taskcoachlib.thirdparty.wxScheduler import wxSCHEDULER_DAILY, \
    wxSCHEDULER_WEEKLY, wxSCHEDULER_MONTHLY, wxSCHEDULER_HORIZONTAL, \
    wxSCHEDULER_VERTICAL


class CalendarConfigDialog(sized_controls.SizedDialog):
    VIEWTYPES = [wxSCHEDULER_DAILY, wxSCHEDULER_WEEKLY, wxSCHEDULER_MONTHLY]
    VIEWORIENTATIONS = [wxSCHEDULER_HORIZONTAL, wxSCHEDULER_VERTICAL]
    VIEWFILTERS = [(False, False, False), (False, True, False), 
                   (True, False, False), (True, True, False), 
                   (True, True, True)]

    def __init__(self, settings, settingsSection, *args, **kwargs):
        self._settings = settings
        self._settingsSection = settingsSection
        super(CalendarConfigDialog, self).__init__(*args, **kwargs)
        pane = self.GetContentsPane()
        pane.SetSizerType('form')
        self.createInterior(pane)
        buttonSizer = self.CreateStdDialogButtonSizer(wx.OK|wx.CANCEL)
        self.SetButtonSizer(buttonSizer)
        self.Fit()
        buttonSizer.GetAffirmativeButton().Bind(wx.EVT_BUTTON, self.ok)

    def createInterior(self, pane):
        self.createPeriodEntry(pane)
        self.createOrientationEntry(pane)
        self.createDisplayEntry(pane)
        self.createLineEntry(pane)
        self.createColorEntry(pane)
        
    def createPeriodEntry(self, pane):
        label = wx.StaticText(pane, 
                              label=_('Kind of period displayed and its count'))
        label.SetSizerProps(valign='center')
        panel = sized_controls.SizedPanel(pane)
        panel.SetSizerType('horizontal')
        self._spanCount = wx.SpinCtrl(panel, value='1', min=1)  # pylint: disable=W0201
        self._spanCount.SetSizerProps(valign='center')
        periods = (_('Day(s)'), _('Week(s)'), _('Month'))
        self._spanType = wx.Choice(panel, choices=periods)  # pylint: disable=W0201
        self._spanType.SetSizerProps(valign='center')
        self._spanCount.SetValue(self._settings.getint(self._settingsSection, 
                                                       'periodcount'))
        selection = self.VIEWTYPES.index(self._settings.getint(self._settingsSection, 
                                                               'viewtype'))
        self._spanType.SetSelection(selection)
        panel.SetSizerProps(valign='center')
        panel.Fit()
        self._spanType.Bind(wx.EVT_CHOICE, self.onChangeViewType)
        
    def createOrientationEntry(self, pane):
        label = wx.StaticText(pane, label=_('Calendar orientation'))
        label.SetSizerProps(valign='center')
        orientations = (_('Horizontal'), _('Vertical'))
        self._orientation = wx.Choice(pane, choices=orientations)  # pylint: disable=W0201
        self._orientation.SetSizerProps(valign='center')
        selection = self.VIEWORIENTATIONS.index(self._settings.getint(self._settingsSection, 'vieworientation'))
        self._orientation.SetSelection(selection)
        
    def createDisplayEntry(self, pane):
        label = wx.StaticText(pane, label=_('Which tasks to display'))
        label.SetSizerProps(valign='center')
        choices = (_('Tasks with a planned start date and a due date'), 
                   _('Tasks with a planned start date'), 
                   _('Tasks with a due date'),
                   _('All tasks, except unplanned tasks'), _('All tasks'))
        self._display = wx.Choice(pane, choices=choices)  # pylint: disable=W0201
        self._display.SetSizerProps(valign='center')
        selection = self.VIEWFILTERS.index((self._settings.getboolean(self._settingsSection, 'shownostart'),
                                            self._settings.getboolean(self._settingsSection, 'shownodue'),
                                            self._settings.getboolean(self._settingsSection, 'showunplanned')))
        self._display.SetSelection(selection)
        
    def createLineEntry(self, pane):
        label = wx.StaticText(pane, 
                              label=_('Draw a line showing the current time'))
        label.SetSizerProps(valign='center')
        self._shownow = wx.CheckBox(pane) # pylint: disable=W0201
        self._shownow.SetSizerProps(valign='center')
        self._shownow.SetValue(self._settings.getboolean(self._settingsSection, 
                                                         'shownow'))

    def createColorEntry(self, pane):
        label = wx.StaticText(pane, 
                              label=_('Color used to highlight the current day'))
        label.SetSizerProps(valign='center')
        hcolor = self._settings.get(self._settingsSection, 'highlightcolor')
        if not hcolor:
            # The highlight color is too dark
            color = wx.SystemSettings.GetColour( wx.SYS_COLOUR_HIGHLIGHT )
            color = wx.Colour(int((color.Red() + 255) / 2),
                              int((color.Green() + 255) / 2),
                              int((color.Blue() + 255) / 2))
        else:
            color = wx.Colour(*tuple(map(int, hcolor.split(','))))  # pylint: disable=W0141
        self._highlight = csel.ColourSelect(pane, size=(100, 20))  # pylint: disable=W0201
        label.SetSizerProps(valign='center')
        self._highlight.SetColour(color)

    def onChangeViewType(self, event): # pylint: disable=W0613
        if self.VIEWTYPES[self._spanType.GetSelection()] == wxSCHEDULER_MONTHLY:
            self._spanCount.SetValue(1)
            self._spanCount.Enable(False)
        else:
            self._spanCount.Enable(True)

    def ok(self, event=None):  # pylint: disable=W0613
        settings, section = self._settings, self._settingsSection
        settings.set(section, 'periodcount', str(self._spanCount.GetValue()))
        settings.set(section, 'viewtype', 
                     str(self.VIEWTYPES[self._spanType.GetSelection()]))
        settings.set(section, 'vieworientation', 
                     str(self.VIEWORIENTATIONS[self._orientation.GetSelection()]))
        shownostart, shownodue, showunplanned = self.VIEWFILTERS[self._display.GetSelection()]
        settings.set(section, 'shownostart', str(shownostart))
        settings.set(section, 'shownodue', str(shownodue))
        settings.set(section, 'showunplanned', str(showunplanned))
        settings.set(section, 'shownow', str(self._shownow.GetValue()))
        color = self._highlight.GetColour()
        settings.set(section, 'highlightcolor', 
                     '%d,%d,%d' % (color.Red(), color.Green(), color.Blue()))
        self.EndModal(wx.ID_OK)




'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
from taskcoachlib.thirdparty.pubsub import pub


class StatusBar(wx.StatusBar):
    def __init__(self, parent, viewer):
        super(StatusBar, self).__init__(parent)
        self.SetFieldsCount(2)
        self.parent = parent
        self.viewer = viewer
        self.__timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.onUpdateStatus, self.__timer)
        pub.subscribe(self.onViewerStatusChanged, 'viewer.status')
        self.scheduledStatusDisplay = None
        self.onViewerStatusChanged()
        self.wxEventTypes = (wx.EVT_MENU_HIGHLIGHT_ALL, wx.EVT_TOOL_ENTER)
        for eventType in self.wxEventTypes:
            parent.Bind(eventType, self.resetStatusBar)

    def resetStatusBar(self, event):
        ''' Unfortunately, the menu's and toolbar don't restore the
            previous statusbar text after they have displayed their help
            text, so we have to do it by hand. '''
        try:
            toolOrMenuId = event.GetSelection() # for CommandEvent from the Toolbar
        except AttributeError:
            toolOrMenuId = event.GetMenuId() # for MenuEvent
        if toolOrMenuId == -1:
            self._displayStatus()
        event.Skip()

    def onViewerStatusChanged(self):
        # Give viewer a chance to update first and only update when the viewer
        # hasn't changed status for 0.5 seconds.
        self.__timer.Start(500, oneShot=True)
              
    def onUpdateStatus(self, event): # pylint: disable=W0613
        if self.__timer:
            self.__timer.Stop()
        self._displayStatus()

    def _displayStatus(self):
        try:
            status1, status2 = self.viewer.statusMessages()
        except AttributeError:
            return # Viewer container contains no viewers 
        super(StatusBar, self).SetStatusText(status1, 0)
        super(StatusBar, self).SetStatusText(status2, 1)

    def SetStatusText(self, message, pane=0, delay=3000): # pylint: disable=W0221
        if self.scheduledStatusDisplay:
            self.scheduledStatusDisplay.Stop()
        super(StatusBar, self).SetStatusText(message, pane)
        self.scheduledStatusDisplay = wx.FutureCall(delay, self._displayStatus)

    def Destroy(self): # pylint: disable=W0221
        for eventType in self.wxEventTypes:
            self.parent.Unbind(eventType)
        if self.scheduledStatusDisplay:
            self.scheduledStatusDisplay.Stop()
        super(StatusBar, self).Destroy()




'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import os, fnmatch
from distutils.command.clean import clean as BaseCleanCommand
from distutils import log


class clean(BaseCleanCommand, object):
    user_options = BaseCleanCommand.user_options + \
        [('really-clean', 'r', 'remove even more files')]
    boolean_options = BaseCleanCommand.boolean_options + ['really-clean']

    def initialize_options(self):
        super(clean, self).initialize_options()
        # pylint: disable=W0201
        self.really_clean = False
        self.cleaning_patterns = ['*.pyc']
        self.really_clean_patterns = ['*.bak']
                    
    def finalize_options(self):
        super(clean, self).finalize_options()
        if self.really_clean:
            self.cleaning_patterns.extend(self.really_clean_patterns)
                    
    def run(self):
        super(clean, self).run()
        if not self.verbose:
            log.info("recursively removing '" + "', '".join(self.cleaning_patterns) + "'")
        for root, _, files in os.walk('.'):
            for pattern in self.cleaning_patterns:
                for filename in fnmatch.filter(files, pattern):
                    filename = os.path.join(root, filename)
                    try:
                        if not self.dry_run:
                            os.unlink(filename)
                        if self.verbose:
                            log.info("removing '%s'"%filename.strip('.\\'))
                    except IOError:
                        pass



# -*- coding: utf-8 -*-

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
Copyright (C) 2008 Rob McMullen <rob.mcmullen@gmail.com>
Copyright (C) 2008 Thomas Sonne Olesen <tpo@sonnet.dk>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import os, wx
from taskcoachlib import command, widgets
from taskcoachlib.domain import attachment
from taskcoachlib.i18n import _
from taskcoachlib.gui import uicommand, menu, dialog 
import base, mixin


class AttachmentViewer(mixin.AttachmentDropTargetMixin, # pylint: disable=W0223
                       base.SortableViewerWithColumns,
                       mixin.SortableViewerForAttachmentsMixin, 
                       mixin.SearchableViewerMixin, mixin.NoteColumnMixin,
                       base.ListViewer): 
    SorterClass = attachment.AttachmentSorter
    viewerImages = base.ListViewer.viewerImages + ['fileopen', 'fileopen_red']

    def __init__(self, *args, **kwargs):
        self.attachments = kwargs.pop('attachmentsToShow')
        kwargs.setdefault('settingssection', 'attachmentviewer')
        super(AttachmentViewer, self).__init__(*args, **kwargs)

    def _addAttachments(self, attachments, item, **itemDialogKwargs):
        # Don't try to add attachments to attachments.
        super(AttachmentViewer, self)._addAttachments(attachments, None, **itemDialogKwargs)

    def domainObjectsToView(self):
        return self.attachments

    def isShowingAttachments(self):
        return True
    
    def curselectionIsInstanceOf(self, class_):
        return class_ == attachment.Attachment

    def createWidget(self):
        imageList = self.createImageList()
        itemPopupMenu = menu.AttachmentPopupMenu(self.parent, self.settings,
            self.presentation(), self)
        columnPopupMenu = menu.ColumnPopupMenu(self)
        self._popupMenus.extend([itemPopupMenu, columnPopupMenu])
        self._columns = self._createColumns()
        widget = widgets.VirtualListCtrl(self, self.columns(), self.onSelect,
            uicommand.Edit(viewer=self),
            itemPopupMenu, columnPopupMenu,
            resizeableColumn=1, **self.widgetCreationKeywordArguments())
        widget.SetColumnWidth(0, 150)
        widget.AssignImageList(imageList, wx.IMAGE_LIST_SMALL)
        return widget

    def _createColumns(self):
        return [widgets.Column('type', _('Type'), 
                               '',
                               width=self.getColumnWidth('type'),
                               imageIndicesCallback=self.typeImageIndices,
                               renderCallback=lambda item: '',
                               resizeCallback=self.onResizeColumn),
                widgets.Column('subject', _('Subject'), 
                               attachment.FileAttachment.subjectChangedEventType(),
                               attachment.URIAttachment.subjectChangedEventType(),
                               attachment.MailAttachment.subjectChangedEventType(), 
                               sortCallback=uicommand.ViewerSortByCommand(viewer=self,
                                   value='subject',
                                   menuText=_('Sub&ject'), helpText=_('Sort by subject')),
                               width=self.getColumnWidth('subject'), 
                               renderCallback=lambda item: item.subject(),
                               resizeCallback=self.onResizeColumn),
                widgets.Column('description', _('Description'),
                               attachment.FileAttachment.descriptionChangedEventType(),
                               attachment.URIAttachment.descriptionChangedEventType(),
                               attachment.MailAttachment.descriptionChangedEventType(),
                               sortCallback=uicommand.ViewerSortByCommand(viewer=self,
                                   value='description',
                                   menuText=_('&Description'), helpText=_('Sort by description')),
                               width=self.getColumnWidth('description'),
                               renderCallback=lambda item: item.description(),
                               resizeCallback=self.onResizeColumn),
                widgets.Column('notes', '', 
                               attachment.FileAttachment.notesChangedEventType(), # pylint: disable=E1101
                               attachment.URIAttachment.notesChangedEventType(), # pylint: disable=E1101
                               attachment.MailAttachment.notesChangedEventType(), # pylint: disable=E1101
                               width=self.getColumnWidth('notes'),
                               alignment=wx.LIST_FORMAT_LEFT,
                               imageIndicesCallback=self.noteImageIndices, # pylint: disable=E1101
                               headerImageIndex=self.imageIndex['note_icon'],
                               renderCallback=lambda item: '',
                               resizeCallback=self.onResizeColumn),
                widgets.Column('creationDateTime', _('Creation date'),
                               width=self.getColumnWidth('creationDateTime'),
                               renderCallback=self.renderCreationDateTime,
                               sortCallback=uicommand.ViewerSortByCommand(viewer=self,
                                                                          value='creationDateTime',
                                                                          menuText=_('&Creation date'),
                                                                          helpText=_('Sort by creation date')),
                               resizeCallback=self.onResizeColumn),
                widgets.Column('modificationDateTime', _('Modification date'),
                               width=self.getColumnWidth('modificationDateTime'),
                               renderCallback=self.renderModificationDateTime,
                               sortCallback=uicommand.ViewerSortByCommand(viewer=self,
                                                                          value='modificationDateTime',
                                                                          menuText=_('&Modification date'),
                                                                          helpText=_('Sort by last modification date')),
                               resizeCallback=self.onResizeColumn,
                               *attachment.Attachment.modificationEventTypes())
                ]

    def createColumnUICommands(self):
        return [\
            uicommand.ToggleAutoColumnResizing(viewer=self,
                                               settings=self.settings),
            None,
            uicommand.ViewColumn(menuText=_('&Description'),
                helpText=_('Show/hide description column'),
                setting='description', viewer=self),
            uicommand.ViewColumn(menuText=_('&Notes'),
                helpText=_('Show/hide notes column'),
                setting='notes', viewer=self),
            uicommand.ViewColumn(menuText=_('&Creation date'),
                helpText=_('Show/hide creation date column'),
                setting='creationDateTime', viewer=self),
            uicommand.ViewColumn(menuText=_('&Modification date'),
                helpText=_('Show/hide last modification date column'),
                setting='modificationDateTime', viewer=self)]
    
    def createCreationToolBarUICommands(self):
        return (uicommand.AttachmentNew(attachments=self.presentation(),
                                        settings=self.settings,
                                        viewer=self),) + \
            super(AttachmentViewer, self).createCreationToolBarUICommands()
        
    def createActionToolBarUICommands(self):
        return (uicommand.AttachmentOpen(attachments=attachment.AttachmentList(),
                                         viewer=self, settings=self.settings),) + \
           super(AttachmentViewer, self).createActionToolBarUICommands()
    
    def typeImageIndices(self, anAttachment, exists=os.path.exists): # pylint: disable=W0613
        if anAttachment.type_ == 'file':
            attachmentBase = self.settings.get('file', 'attachmentbase')
            if exists(anAttachment.normalizedLocation(attachmentBase)):
                index = self.imageIndex['fileopen']
            else:
                index = self.imageIndex['fileopen_red']
        else:
            try:
                index = self.imageIndex[{'uri': 'earth_blue_icon',
                                         'mail': 'envelope_icon'}[anAttachment.type_]]
            except KeyError:
                index = -1
        return {wx.TreeItemIcon_Normal: index}

    def itemEditorClass(self):
        return dialog.editor.AttachmentEditor

    def newItemCommandClass(self):
        raise NotImplementedError  # pragma: no cover
     
    def newSubItemCommandClass(self):
        return None

    def deleteItemCommandClass(self):
        raise NotImplementedError  # pragma: no cover

    def cutItemCommandClass(self):
        raise NotImplementedError  # pragma: no cover



# -*- coding: utf-8 -*-

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
import test
import weakref
from taskcoachlib import patterns
from taskcoachlib.domain import base, date


class SynchronizedObjectTest(test.TestCase):
    def setUp(self):
        self.object = base.SynchronizedObject()
        self.events = []
        
    def onEvent(self, event):
        self.events.append(event)
        
    def registerObserver(self, eventType):  # pylint: disable=W0221
        patterns.Publisher().registerObserver(self.onEvent, eventType)
        
    def assertObjectStatus(self, expectedStatus):
        self.assertEqual(expectedStatus, self.object.getStatus())
        
    def assertOneEventReceived(self, eventSource, eventType, *values):
        self.assertEqual([patterns.Event(eventType, eventSource, *values)], 
                         self.events)
    
    def testInitialStatus(self):
        self.assertObjectStatus(base.SynchronizedObject.STATUS_NEW)
                         
    def testMarkDeleted(self):
        self.object.markDeleted()
        self.assertObjectStatus(base.SynchronizedObject.STATUS_DELETED)
                         
    def testMarkDeletedNotification(self):
        self.registerObserver(self.object.markDeletedEventType())
        self.object.markDeleted()
        self.assertOneEventReceived(self.object,
            self.object.markDeletedEventType(), self.object.getStatus())
    
    def testMarkNewObjectAsNotDeleted(self):
        self.object.cleanDirty()
        self.assertObjectStatus(base.SynchronizedObject.STATUS_NONE)
    
    def testMarkDeletedObjectAsUndeleted(self):
        self.object.markDeleted()
        self.object.cleanDirty()
        self.assertObjectStatus(base.SynchronizedObject.STATUS_NONE) 

    def testMarkNotDeletedNotification(self):
        self.object.markDeleted()
        self.registerObserver(self.object.markNotDeletedEventType())
        self.object.cleanDirty()
        self.assertOneEventReceived(self.object, 
            self.object.markNotDeletedEventType(), self.object.getStatus()) 

    def testSetStateToDeletedCausesNotification(self):
        self.object.markDeleted()
        state = self.object.__getstate__()
        self.object.cleanDirty()
        self.registerObserver(self.object.markDeletedEventType())
        self.object.__setstate__(state)                
        self.assertOneEventReceived(self.object, 
            self.object.markDeletedEventType(), self.object.STATUS_DELETED)

    def testSetStateToNotDeletedCausesNotification(self):
        state = self.object.__getstate__()
        self.object.markDeleted()
        self.registerObserver(self.object.markNotDeletedEventType())
        self.object.__setstate__(state)                
        self.assertOneEventReceived(self.object, 
            self.object.markNotDeletedEventType(), self.object.STATUS_NEW)
                    
                    
class ObjectSubclass(base.Object):
    pass


class ObjectTest(test.TestCase):
    def setUp(self):
        self.object = base.Object()
        self.subclassObject = ObjectSubclass()
        self.eventsReceived = []
        for eventType in (self.object.subjectChangedEventType(), 
                          self.object.descriptionChangedEventType(),
                          self.object.appearanceChangedEventType()):
            patterns.Publisher().registerObserver(self.onEvent, eventType)

    def onEvent(self, event):
        self.eventsReceived.append(event)

    # Basic tests:

    def testCyclicReference(self):
        domainObject = base.Object()
        weak = weakref.ref(domainObject)
        del domainObject # Assuming CPython
        self.failUnless(weak() is None)

    # Custom attributes tests:

    def testCustomAttributes(self):
        self.object.setDescription('\n[mailto:cc=foo@bar.com]\n[mailto:cc=baz@spam.com]\n')
        self.assertEqual(self.object.customAttributes('mailto'), set(['cc=foo@bar.com', 'cc=baz@spam.com']))

    # Id tests:
        
    def testSetIdOnCreation(self):
        domainObject = base.Object(id='123')
        self.assertEqual('123', domainObject.id())
        
    def testIdIsAString(self):
        self.assertEqual(type(''), type(self.object.id()))
        
    def testDifferentObjectsHaveDifferentIds(self):
        self.assertNotEqual(base.Object().id(), self.object.id())
        
    def testCopyHasDifferentId(self):
        objectId = self.object.id()  # Force generation of id
        copy = self.object.copy()
        self.assertNotEqual(copy.id(), objectId)
    
    # Creation date/time tests:
    
    def testSetCreationDateTimeOnCreation(self):
        creation_datetime = date.DateTime(2012, 12, 12, 10, 0, 0)
        domain_object = base.Object(creationDateTime=creation_datetime)
        self.assertEqual(creation_datetime, domain_object.creationDateTime())
        
    def testCreationDateTimeIsSetWhenNotPassed(self):
        now = date.Now()
        creation_datetime = self.object.creationDateTime()
        minute = date.TimeDelta(seconds=60)
        self.failUnless(now - minute < creation_datetime < now + minute)
        
    # Modification date/time tests:
    
    def testSetModificationDateTimeOnCreation(self):
        modification_datetime = date.DateTime(2012, 12, 12, 10, 0, 0)
        domain_object = base.Object(modificationDateTime=modification_datetime)
        self.assertEqual(modification_datetime, 
                         domain_object.modificationDateTime())
        
    def testModificationDateTimeIsNotSetWhenNotPassed(self):
        self.assertEqual(date.DateTime.min, self.object.modificationDateTime())
        
    # Subject tests:
        
    def testSubjectIsEmptyByDefault(self):
        self.assertEqual('', self.object.subject())
        
    def testSetSubjectOnCreation(self):
        domainObject = base.Object(subject='Hi')
        self.assertEqual('Hi', domainObject.subject())
        
    def testSetSubject(self):
        self.object.setSubject('New subject')
        self.assertEqual('New subject', self.object.subject())
        
    def testSetSubjectCausesNotification(self):
        self.object.setSubject('New subject')
        self.assertEqual(patterns.Event( \
            self.object.subjectChangedEventType(), self.object, 'New subject'), 
            self.eventsReceived[0])
        
    def testSetSubjectUnchangedDoesNotCauseNotification(self):
        self.object.setSubject('')
        self.failIf(self.eventsReceived)
        
    def testSubjectChangedNotificationIsDifferentForSubclass(self):
        self.subclassObject.setSubject('New')
        self.failIf(self.eventsReceived)
        
    # Description tests:
    
    def testDescriptionIsEmptyByDefault(self):
        self.failIf(self.object.description())
        
    def testSetDescriptionOnCreation(self):
        domainObject = base.Object(description='Hi')
        self.assertEqual('Hi', domainObject.description())
        
    def testSetDescription(self):
        self.object.setDescription('New description')
        self.assertEqual('New description', self.object.description())
        
    def testSetDescriptionCausesNotification(self):
        self.object.setDescription('New description')
        self.assertEqual(patterns.Event( \
            self.object.descriptionChangedEventType(), self.object, 
            'New description'), 
            self.eventsReceived[0])

    def testSetDescriptionUnchangedDoesNotCauseNotification(self):
        self.object.setDescription('')
        self.failIf(self.eventsReceived)

    def testDescriptionChangedNotificationIsDifferentForSubclass(self):
        self.subclassObject.setDescription('New')
        self.failIf(self.eventsReceived)
            
    # State tests:
    
    def testGetState(self):
        self.assertEqual(dict(subject='', description='', id=self.object.id(),
                              status=self.object.getStatus(), fgColor=None,
                              bgColor=None, font=None, icon='', selectedIcon='', 
                              creationDateTime=self.object.creationDateTime(),
                              modificationDateTime=self.object.modificationDateTime(),
                              ordering=self.object.ordering()),
                         self.object.__getstate__())

    def testSetState(self):
        newState = dict(subject='New', description='New', id=None,
                        status=self.object.STATUS_DELETED, 
                        fgColor=wx.GREEN, bgColor=wx.RED, font=wx.SWISS_FONT,
                        icon='icon', selectedIcon='selectedIcon',
                        creationDateTime=date.DateTime(2012, 12, 12, 12, 0, 0),
                        modificationDateTime=date.DateTime(2012, 12, 12, 12, 1, 0),
                        ordering=42L)
        self.object.__setstate__(newState)
        self.assertEqual(newState, self.object.__getstate__())
        
    def testSetState_SendsOneNotification(self):
        newState = dict(subject='New', description='New', id=None,
                        status=self.object.STATUS_DELETED, 
                        fgColor=wx.GREEN, bgColor=wx.RED, font=wx.SWISS_FONT,
                        icon='icon', selectedIcon='selectedIcon',
                        creationDateTime=date.DateTime(2013, 1, 1, 0, 0, 0),
                        modificationDateTime=date.DateTime(2013, 1, 1, 1, 0, 0),
                        ordering=42L)
        self.object.__setstate__(newState)
        self.assertEqual(1, len(self.eventsReceived))
        
    # Copy tests:
    
    def testCopy_IdIsNotCopied(self):
        copy = self.object.copy()
        self.assertNotEqual(copy.id(), self.object.id())
    
    def testCopy_CreationDateTimeIsNotCopied(self):
        copy = self.object.copy()
        # Use >= to prevent failures on fast computers with low time granularity
        self.failUnless(copy.creationDateTime() >= self.object.creationDateTime())
        
    def testCopy_ModificationDateTimeIsNotCopied(self):
        self.object.setModificationDateTime(date.DateTime(2013, 1, 1, 1, 0, 0))
        copy = self.object.copy()
        self.assertEqual(date.DateTime.min, copy.modificationDateTime())
        
    def testCopy_SubjectIsCopied(self):
        self.object.setSubject('New subject')
        copy = self.object.copy()
        self.assertEqual(copy.subject(), self.object.subject())

    def testCopy_DescriptionIsCopied(self):
        self.object.setDescription('New description')
        copy = self.object.copy()
        self.assertEqual(copy.description(), self.object.description())

    def testCopy_ForegroundColorIsCopied(self):
        self.object.setForegroundColor(wx.RED)
        copy = self.object.copy()
        self.assertEqual(copy.foregroundColor(), self.object.foregroundColor())

    def testCopy_BackgroundColorIsCopied(self):
        self.object.setBackgroundColor(wx.RED)
        copy = self.object.copy()
        self.assertEqual(copy.backgroundColor(), self.object.backgroundColor())
        
    def testCopy_FontIsCopied(self):
        self.object.setFont(wx.SWISS_FONT)
        copy = self.object.copy()
        self.assertEqual(copy.font(), self.object.font())

    def testCopy_IconIsCopied(self):
        self.object.setIcon('icon')
        copy = self.object.copy()
        self.assertEqual(copy.icon(), self.object.icon())
        
    def testCopy_ShouldUseSubclassForCopy(self):
        copy = self.subclassObject.copy()
        self.assertEqual(copy.__class__, self.subclassObject.__class__)

    # Color tests
    
    def testDefaultForegroundColor(self):
        self.assertEqual(None, self.object.foregroundColor())
        
    def testSetForegroundColor(self):
        self.object.setForegroundColor(wx.GREEN)
        self.assertEqual(wx.GREEN, self.object.foregroundColor())
    
    def testSetForegroundColorWithTupleColor(self):
        self.object.setForegroundColor((255, 0, 0, 255))
        self.assertEqual(wx.RED, self.object.foregroundColor())

    def testSetForegroundColorOnCreation(self):
        domainObject = base.Object(fgColor=wx.GREEN)
        self.assertEqual(wx.GREEN, domainObject.foregroundColor())
    
    def testForegroundColorChangedNotification(self):
        self.object.setForegroundColor(wx.BLACK)
        self.assertEqual(1, len(self.eventsReceived))

    def testDefaultBackgroundColor(self):
        self.assertEqual(None, self.object.backgroundColor())
    
    def testSetBackgroundColor(self):
        self.object.setBackgroundColor(wx.RED)
        self.assertEqual(wx.RED, self.object.backgroundColor())

    def testSetBackgroundColorWithTupleColor(self):
        self.object.setBackgroundColor((255, 0, 0, 255))
        self.assertEqual(wx.RED, self.object.backgroundColor())

    def testSetBackgroundColorOnCreation(self):
        domainObject = base.Object(bgColor=wx.GREEN)
        self.assertEqual(wx.GREEN, domainObject.backgroundColor())
    
    def testBackgroundColorChangedNotification(self):
        self.object.setBackgroundColor(wx.BLACK)
        self.assertEqual(1, len(self.eventsReceived))      
        
    # Font tests:
    
    def testDefaultFont(self):
        self.assertEqual(None, self.object.font())
        
    def testSetFont(self):
        self.object.setFont(wx.SWISS_FONT)
        self.assertEqual(wx.SWISS_FONT, self.object.font())

    def testSetFontOnCreation(self):
        domainObject = base.Object(font=wx.SWISS_FONT)
        self.assertEqual(wx.SWISS_FONT, domainObject.font())

    def testFontChangedNotification(self):
        self.object.setFont(wx.SWISS_FONT)
        self.assertEqual(1, len(self.eventsReceived))

    # Icon tests:

    def testDefaultIcon(self):
        self.assertEqual('', self.object.icon())

    def testSetIcon(self):
        self.object.setIcon('icon')
        self.assertEqual('icon', self.object.icon())

    def testSetIconOnCreation(self):
        domainObject = base.Object(icon='icon')
        self.assertEqual('icon', domainObject.icon())

    def testIconChangedNotification(self):
        self.object.setIcon('icon')
        self.assertEqual(1, len(self.eventsReceived))

    def testDefaultSelectedIcon(self):
        self.assertEqual('', self.object.selectedIcon())

    def testSetSelectedIcon(self):
        self.object.setSelectedIcon('selected')
        self.assertEqual('selected', self.object.selectedIcon())

    def testSelectedIconAfterSettingRegularIconOnly(self):
        self.object.setIcon('icon')
        self.assertEqual('', self.object.selectedIcon())

    def testSetSelectedIconOnCreation(self):
        domainObject = base.Object(selectedIcon='icon')
        self.assertEqual('icon', domainObject.selectedIcon())

    def testSelectedIconChangedNotification(self):
        self.object.setSelectedIcon('icon')
        self.assertEqual(1, len(self.eventsReceived))

    # Event types:
    
    def testModificationEventTypes(self):
        self.assertEqual([self.object.subjectChangedEventType(),
                          self.object.descriptionChangedEventType(),
                          self.object.appearanceChangedEventType(),
                          self.object.orderingChangedEventType()],
                         self.object.modificationEventTypes())


class CompositeObjectTest(test.TestCase):
    def setUp(self):
        self.compositeObject = base.CompositeObject()
        self.child = None
        self.eventsReceived = []
        
    def onEvent(self, event):
        self.eventsReceived.append(event)
        
    def addChild(self, **kwargs):
        self.child = base.CompositeObject(**kwargs)
        self.compositeObject.addChild(self.child)
        self.child.setParent(self.compositeObject)
        
    def removeChild(self):
        self.compositeObject.removeChild(self.child)

    def testIsExpanded(self):
        self.failIf(self.compositeObject.isExpanded())
        
    def testExpand(self):
        self.compositeObject.expand()
        self.failUnless(self.compositeObject.isExpanded())
        
    def testCollapse(self):
        self.compositeObject.expand()
        self.compositeObject.expand(False)
        self.failIf(self.compositeObject.isExpanded())
        
    def testSetExpansionStateViaConstructor(self):
        compositeObject = base.CompositeObject(expandedContexts=['None'])
        self.failUnless(compositeObject.isExpanded())

    def testSetExpansionStatesViaConstructor(self):
        compositeObject = base.CompositeObject(expandedContexts=['context1',
            'context2'])
        self.assertEqual(['context1', 'context2'],
                         sorted(compositeObject.expandedContexts()))

    def testExpandInContext_DoesNotChangeExpansionStateInDefaultContext(self):
        self.compositeObject.expand(context='some_viewer')
        self.failIf(self.compositeObject.isExpanded())

    def testExpandInContext_DoesChangeExpansionStateInGivenContext(self):
        self.compositeObject.expand(context='some_viewer')
        self.failUnless(self.compositeObject.isExpanded(context='some_viewer'))

    def testIsExpandedInUnknownContext_ReturnsFalse(self):
        self.failIf(self.compositeObject.isExpanded(context='whatever'))

    def testGetContextsWhereExpanded(self):
        self.assertEqual([], self.compositeObject.expandedContexts())
        
    def testRecursiveSubject(self):
        self.compositeObject.setSubject('parent')
        self.addChild(subject='child')
        self.assertEqual(u'parent -> child', self.child.subject(recursive=True))
        
    def testSubjectNotification(self):
        self.addChild(subject='child')
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=self.compositeObject.subjectChangedEventType(),
            eventSource=self.child)
        self.compositeObject.setSubject('parent')
        self.assertEqual([patterns.Event(self.compositeObject.subjectChangedEventType(),
                                         self.child, 'child')],
                         self.eventsReceived)

    def testSubItemUsesParentForegroundColor(self):
        self.addChild()
        self.compositeObject.setForegroundColor(wx.RED)
        self.assertEqual(wx.RED, self.child.foregroundColor(recursive=True))

    def testSubItemDoesNotUseParentForegroundColorIfItHasItsOwnForegroundColor(self):
        self.addChild(fgColor=wx.RED)
        self.compositeObject.setForegroundColor(wx.BLUE)        
        self.assertEqual(wx.RED, self.child.foregroundColor(recursive=True))

    def testApperanceChangedNotificationWhenForegroundColorChanges(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.appearanceChangedEventType(),
            eventSource=self.child)
        self.compositeObject.setForegroundColor(wx.RED)
        self.assertEqual(1, len(self.eventsReceived))

    def testSubItemUsesParentBackgroundColor(self):
        self.addChild()
        self.compositeObject.setBackgroundColor(wx.RED)
        self.assertEqual(wx.RED, self.child.backgroundColor(recursive=True))
        
    def testSubItemDoesNotUseParentBackgroundColorIfItHasItsOwnBackgroundColor(self):
        self.addChild(bgColor=wx.RED)
        self.compositeObject.setBackgroundColor(wx.BLUE)        
        self.assertEqual(wx.RED, self.child.backgroundColor(recursive=True))
        
    def testBackgroundColorChangedNotification(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.appearanceChangedEventType(),
            eventSource=self.child)
        self.compositeObject.setBackgroundColor(wx.RED)
        self.assertEqual(1, len(self.eventsReceived))
        
    def testSubItemUsesParentFont(self):
        self.addChild()
        self.compositeObject.setFont(wx.ITALIC_FONT)
        self.assertEqual(wx.ITALIC_FONT, self.child.font(recursive=True))
        
    def testSubItemDoesNotUseParentFontIfItHasItsOwnFont(self):
        self.addChild(font=wx.SWISS_FONT)
        self.compositeObject.setFont(wx.ITALIC_FONT)
        self.assertEqual(wx.SWISS_FONT, self.child.font(recursive=True))
        
    def testFontChangedNotification(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.appearanceChangedEventType(), 
            eventSource=self.child)
        self.compositeObject.setFont(wx.SWISS_FONT)
        self.assertEqual(1, len(self.eventsReceived))

    def testSubItemUsesParentIcon(self):
        self.addChild()
        self.compositeObject.setIcon('icon')
        self.assertEqual('icon', self.child.icon(recursive=True))

    def testSubItemDoesNotUseParentIconIfItHasItsOwnIcon(self):
        self.addChild(icon='childIcon')
        self.compositeObject.setIcon('icon')
        self.assertEqual('childIcon', self.child.icon(recursive=True))

    def testIconChangedNotification(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.appearanceChangedEventType(),
            eventSource=self.child)
        self.compositeObject.setIcon('icon')
        self.assertEqual(1, len(self.eventsReceived))

    def testSubItemUsesParentSelectedIcon(self):
        self.addChild()
        self.compositeObject.setSelectedIcon('icon')
        self.assertEqual('icon', self.child.selectedIcon(recursive=True))

    def testSubItemDoesNotUseParentSelectedIconIfItHasItsOwnSelectedIcon(self):
        self.addChild(selectedIcon='childIcon')
        self.compositeObject.setSelectedIcon('icon')
        self.assertEqual('childIcon', self.child.selectedIcon(recursive=True))

    def testSubItemUsesParentSelectedIconEvenIfItHasItsOwnIcon(self):
        self.addChild(icon='childIcon')
        self.compositeObject.setSelectedIcon('icon')
        self.assertEqual('icon', self.child.selectedIcon(recursive=True))

    def testSelectedIconChangedNotification(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.appearanceChangedEventType(),
            eventSource=self.child)
        self.compositeObject.setSelectedIcon('icon')
        self.assertEqual(1, len(self.eventsReceived))

    def testCompositeWithChildrenUsesPluralIconIfAvailable(self):
        self.compositeObject.setIcon('book_icon')
        self.assertEqual('book_icon', self.compositeObject.icon(recursive=True))
        self.addChild()
        self.assertEqual('books_icon', self.compositeObject.icon(recursive=True))
        self.assertEqual('book_icon', self.compositeObject.icon(recursive=False))

    def testCompositeWithChildrenUsesPluralSelectedIconIfAvailable(self):
        self.compositeObject.setSelectedIcon('book_icon')
        self.assertEqual('book_icon', self.compositeObject.selectedIcon(recursive=True))
        self.addChild()
        self.assertEqual('books_icon', self.compositeObject.selectedIcon(recursive=True))
        self.assertEqual('book_icon', self.compositeObject.selectedIcon(recursive=False)) 

    def testCompositeWithoutChildrenDoesNotUseSingularIconIfAvailable(self):
        self.compositeObject.setIcon('books_icon')
        self.assertEqual('books_icon', self.compositeObject.icon(recursive=False))
        self.assertEqual('books_icon', self.compositeObject.icon(recursive=True))

    def testCompositeWithoutChildrenDoesNotUseSingularSelectedIconIfAvailable(self):
        self.compositeObject.setSelectedIcon('books_icon')
        self.assertEqual('books_icon', self.compositeObject.selectedIcon(recursive=False))
        self.assertEqual('books_icon', self.compositeObject.selectedIcon(recursive=True))

    def testChildOfCompositeUsesSingularIconIfAvailable(self):
        self.compositeObject.setIcon('books_icon')
        self.addChild()
        self.assertEqual('book_icon', self.child.icon(recursive=True))

    def testChildOfCompositeUsesSingularSelectedIconIfAvailable(self):
        self.compositeObject.setSelectedIcon('books_icon')
        self.addChild()
        self.assertEqual('book_icon', self.child.selectedIcon(recursive=True))

    def testParentUsesSingularIconAfterChildRemoved(self):
        self.compositeObject.setIcon('book_icon')
        self.addChild()
        self.assertEqual('books_icon', self.compositeObject.icon(recursive=True))
        self.removeChild()
        self.assertEqual('book_icon', self.compositeObject.icon(recursive=True))

    def testParentUsesSingularSelectedIconAfterChildRemoved(self):
        self.compositeObject.setSelectedIcon('book_icon')
        self.addChild()
        self.assertEqual('books_icon', self.compositeObject.selectedIcon(recursive=True))
        self.removeChild()
        self.assertEqual('book_icon', self.compositeObject.selectedIcon(recursive=True))

    def testCopy(self):
        self.compositeObject.expand(context='some_viewer')
        copy = self.compositeObject.copy()
        # pylint: disable=E1101
        self.assertEqual(copy.expandedContexts(),
                         self.compositeObject.expandedContexts())
        self.compositeObject.expand(context='another_viewer')
        self.failIf('another_viewer' in copy.expandedContexts())
        
    def testMarkDeleted(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.markDeletedEventType())
        self.compositeObject.markDeleted()
        expectedEvent = patterns.Event(base.CompositeObject.markDeletedEventType(),
                                       self.compositeObject, base.CompositeObject.STATUS_DELETED)
        expectedEvent.addSource(self.child, base.CompositeObject.STATUS_DELETED)
        self.assertEqual([expectedEvent], self.eventsReceived)
        
    def testMarkDirty(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.markNotDeletedEventType())
        self.compositeObject.markDeleted()
        self.compositeObject.markDirty(force=True)
        expectedEvent = patterns.Event(base.CompositeObject.markNotDeletedEventType(),
                                       self.compositeObject, base.CompositeObject.STATUS_CHANGED)
        expectedEvent.addSource(self.child, base.CompositeObject.STATUS_CHANGED)
        self.assertEqual([expectedEvent], self.eventsReceived)

    def testMarkNew(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.markNotDeletedEventType())
        self.compositeObject.markDeleted()
        self.compositeObject.markNew()
        expectedEvent = patterns.Event(base.CompositeObject.markNotDeletedEventType(),
                                       self.compositeObject, base.CompositeObject.STATUS_NEW)
        expectedEvent.addSource(self.child, base.CompositeObject.STATUS_NEW)
        self.assertEqual([expectedEvent], self.eventsReceived)
        
    def testCleanDirty(self):
        self.addChild()
        patterns.Publisher().registerObserver(self.onEvent,
            eventType=base.CompositeObject.markNotDeletedEventType())
        self.compositeObject.markDeleted()
        self.compositeObject.cleanDirty()
        expectedEvent = patterns.Event(base.CompositeObject.markNotDeletedEventType(),
                                       self.compositeObject, base.CompositeObject.STATUS_NONE)
        expectedEvent.addSource(self.child, base.CompositeObject.STATUS_NONE)
        self.assertEqual([expectedEvent], self.eventsReceived)
        
    def testModificationEventTypes(self):
        self.assertEqual([self.compositeObject.addChildEventType(),
                          self.compositeObject.removeChildEventType(),
                          self.compositeObject.subjectChangedEventType(),
                          self.compositeObject.descriptionChangedEventType(),
                          self.compositeObject.appearanceChangedEventType(),
                          self.compositeObject.orderingChangedEventType(),
                          self.compositeObject.expansionChangedEventType()], 
                         self.compositeObject.modificationEventTypes())


class BaseCollectionTest(test.TestCase):
    def setUp(self):
        self.collection = base.Collection()
        
    def testLookupByIdWhenCollectionIsEmptyRaisesIndexError(self):
        self.assertRaises(IndexError, self.collection.getObjectById, 'id')
        
    def testLookupIdWhenObjectIsInCollection(self):
        domainObject = base.CompositeObject()
        self.collection.append(domainObject)
        self.assertEqual(domainObject, 
                         self.collection.getObjectById(domainObject.id()))



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test, wx
from taskcoachlib import patterns
from taskcoachlib.domain import category, categorizable, date



class CategorizableCompositeObjectTest(test.TestCase):
    def setUp(self):
        self.categorizable = categorizable.CategorizableCompositeObject(subject='categorizable')
        self.category = category.Category('category')
        
    categoryAddedEventType = categorizable.CategorizableCompositeObject.categoryAddedEventType()
    categoryRemovedEventType = categorizable.CategorizableCompositeObject.categoryRemovedEventType()
    categorySubjectChangedEventType = categorizable.CategorizableCompositeObject.categorySubjectChangedEventType()
    appearanceChangedEventType = categorizable.CategorizableCompositeObject.appearanceChangedEventType()
        
    def assertEvent(self, *expectedEventArgs):
        expectedEvent = patterns.Event(*expectedEventArgs)
        self.assertEqual([expectedEvent], self.events)
        
    def testCategorizableDoesNotBelongToAnyCategoryByDefault(self):
        for recursive in False, True:
            for upwards in False, True:
                self.failIf(self.categorizable.categories(recursive=recursive,
                                                          upwards=upwards))

    def testCategorizableHasNoForegroundColorByDefault(self):
        self.assertEqual(None, self.categorizable.foregroundColor())
    
    def testCategorizableHasNoBackgroundColorByDefault(self):
        self.assertEqual(None, self.categorizable.backgroundColor())

    def testCategorizableHasNoFontByDefault(self):
        self.assertEqual(None, self.categorizable.font())
        
    def testAddCategory(self):
        self.categorizable.addCategory(self.category)
        self.assertEqual(set([self.category]), self.categorizable.categories())

    def testAddCategoryNotification(self):
        self.registerObserver(self.categoryAddedEventType)
        self.categorizable.addCategory(self.category)
        self.assertEvent(self.categoryAddedEventType, self.categorizable, 
                         self.category) 
        
    def testAddSecondCategory(self):
        self.categorizable.addCategory(self.category)
        cat2 = category.Category('category 2')
        self.categorizable.addCategory(cat2)
        self.assertEqual(set([self.category, cat2]), 
            self.categorizable.categories())
        
    def testAddSameCategoryTwice(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.addCategory(self.category)
        self.assertEqual(set([self.category]), self.categorizable.categories())
        
    def testAddSameCategoryTwiceCausesNoNotification(self):
        self.categorizable.addCategory(self.category)
        self.registerObserver(self.categoryAddedEventType)
        self.categorizable.addCategory(self.category)
        self.failIf(self.events)
    
    def testAddCategoryViaConstructor(self):
        categorizableObject = categorizable.CategorizableCompositeObject(categories=[self.category])
        self.assertEqual(set([self.category]), categorizableObject.categories())
        
    def testAddCategoriesViaConstructor(self):
        anotherCategory = category.Category('Another category')
        categories = [self.category, anotherCategory]
        categorizableObject = categorizable.CategorizableCompositeObject(categories= \
            categories)
        self.assertEqual(set(categories), categorizableObject.categories())
        
    def testAddCategoryDoesNotAddCategorizableToCategory(self):
        self.categorizable.addCategory(self.category)
        self.assertEqual(set([]), self.category.categorizables())
        
    def testAddParentToCategory(self):
        child = categorizable.CategorizableCompositeObject(subject='child')
        self.registerObserver(self.categoryAddedEventType, eventSource=child)
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        cat = category.Category(subject='Parent category')
        self.categorizable.addCategory(cat)
        self.assertEvent(self.categoryAddedEventType, child, cat)
        
    def testRemoveCategory(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(set(), self.categorizable.categories())
        
    def testRemoveCategoryNotification(self):
        self.categorizable.addCategory(self.category)
        self.registerObserver(self.categoryRemovedEventType)
        self.categorizable.removeCategory(self.category)
        self.assertEvent(self.categoryRemovedEventType, self.categorizable,
                         self.category)

    def testRemoveCategoryTwice(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.removeCategory(self.category)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(set(), self.categorizable.categories())

    def testRemoveCategoryTwiceNotification(self):
        self.categorizable.addCategory(self.category)
        self.registerObserver(self.categoryRemovedEventType)
        self.categorizable.removeCategory(self.category)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(1, len(self.events))
        
    def testCategorySubjectChanged(self):
        self.registerObserver(self.categorySubjectChangedEventType)
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.category.setSubject('New subject')
        self.assertEvent(self.categorySubjectChangedEventType, 
                         self.categorizable, 'New subject')

    def testCategorySubjectChanged_NotifySubItemsToo(self):
        childCategorizable = categorizable.CategorizableCompositeObject(subject='Child categorizable')
        self.registerObserver(self.categorySubjectChangedEventType, eventSource=childCategorizable)
        self.categorizable.addChild(childCategorizable)
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.category.setSubject('New subject')
        self.assertEvent(self.categorySubjectChangedEventType, 
                         childCategorizable, 'New subject') 

    def testForegroundColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setForegroundColor(wx.RED)
        self.assertEqual(wx.RED, self.categorizable.foregroundColor(recursive=True))

    def testBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setBackgroundColor(wx.RED)
        self.assertEqual(wx.RED, self.categorizable.backgroundColor(recursive=True))
        
    def testFont(self):
        self.categorizable.addCategory(self.category)
        self.category.setFont(wx.SWISS_FONT)
        self.assertEqual(wx.SWISS_FONT, self.categorizable.font(recursive=True))

    def testCategorizableOwnForegroundColorOverridesCategoryForegroundColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setForegroundColor(wx.RED)
        self.categorizable.setForegroundColor(wx.GREEN)
        self.assertEqual(wx.GREEN, self.categorizable.foregroundColor(recursive=True))

    def testCategorizableOwnBackgroundColorOverridesCategoryBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setBackgroundColor(wx.RED)
        self.categorizable.setBackgroundColor(wx.GREEN)
        self.assertEqual(wx.GREEN, self.categorizable.backgroundColor(recursive=True))

    def testCategorizableOwnFontOverridesCategoryFont(self):
        self.categorizable.addCategory(self.category)
        self.category.setFont(wx.SWISS_FONT)
        self.categorizable.setFont(wx.NORMAL_FONT)
        self.assertEqual(wx.NORMAL_FONT, self.categorizable.font())
        
    def testForegroundColorWithTupleColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setForegroundColor((255, 0, 0, 255))
        self.assertEqual(wx.RED, self.categorizable.foregroundColor(recursive=True))
        
    def testBackgroundColorWithTupleColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setBackgroundColor((255, 0, 0, 255))
        self.assertEqual(wx.RED, self.categorizable.backgroundColor(recursive=True))

    def testSubItemUsesParentForegroundColor(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        self.category.setForegroundColor(wx.RED)
        self.assertEqual(wx.RED, child.foregroundColor(recursive=True))
    
    def testSubItemUsesParentBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        self.category.setBackgroundColor(wx.RED)
        self.assertEqual(wx.RED, child.backgroundColor(recursive=True))
        
    def testSubItemUsesParentFont(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        self.category.setFont(wx.SWISS_FONT)
        self.assertEqual(wx.SWISS_FONT, child.font(recursive=True))

    def testSubItemDoesNotUseParentForegroundColorWhenItHasItsOwnForegroundColor(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        child.addCategory(self.category)
        self.categorizable.setForegroundColor(wx.RED)
        self.category.setForegroundColor(wx.BLUE)
        self.assertEqual(wx.BLUE, child.foregroundColor(recursive=True))
        
    def testSubItemDoesNotUseParentBackgroundColorWhenItHasItsOwnBackgroundColor(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        child.addCategory(self.category)
        self.categorizable.setBackgroundColor(wx.RED)
        self.category.setBackgroundColor(wx.BLUE)
        self.assertEqual(wx.BLUE, child.backgroundColor(recursive=True))

    def testSubItemDoesNotUseParentFontWhenItHasItsOwnFont(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        child.setParent(self.categorizable)
        child.addCategory(self.category)
        self.categorizable.setFont(wx.Font(10, wx.FONTFAMILY_SWISS, 
            wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        categoryFont = wx.Font(11, wx.FONTFAMILY_ROMAN, 
            wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        self.category.setFont(categoryFont)
        self.assertEqual(categoryFont, child.font(recursive=True))

    def testForegroundColorChanged(self):
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.registerObserver(self.appearanceChangedEventType)
        self.category.setForegroundColor(wx.RED)
        self.assertEqual(1, len(self.events))
    
    def testBackgroundColorChanged(self):
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.registerObserver(self.appearanceChangedEventType)
        self.category.setBackgroundColor(wx.RED)
        self.assertEqual(1, len(self.events))
        
    def testFontChanged(self):
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.registerObserver(self.appearanceChangedEventType)
        self.category.setFont(wx.SWISS_FONT)
        self.assertEqual(1, len(self.events))

    def testIconChanged(self):
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.registerObserver(self.appearanceChangedEventType)
        self.category.setIcon('icon')
        self.assertEqual(1, len(self.events))

    def testForegroundColorChanged_NotifySubItemsToo(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.registerObserver(self.appearanceChangedEventType, eventSource=child)
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.category.setForegroundColor(wx.RED)
        self.assertEqual(1, len(self.events))

    def testBackgroundColorChanged_NotifySubItemsToo(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.registerObserver(self.appearanceChangedEventType, 
                              eventSource=child)
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.category.setBackgroundColor(wx.RED)
        self.assertEqual(1, len(self.events))

    def testFontChanged_NotifySubItemsToo(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.registerObserver(self.appearanceChangedEventType, eventSource=child)
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.category.setFont(wx.SWISS_FONT)
        self.assertEqual(1, len(self.events))

    def testIconChanged_NotifySubItemsToo(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.registerObserver(self.appearanceChangedEventType, eventSource=child)
        self.categorizable.addCategory(self.category)
        self.category.addCategorizable(self.categorizable)
        self.category.setIcon('icon')
        self.assertEqual(1, len(self.events))

    def testCategorizableDoesNotNotifyWhenItHasItsOwnForegroundColor(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.setForegroundColor(wx.RED)
        self.registerObserver(self.categorizable.appearanceChangedEventType())
        self.category.setForegroundColor(wx.GREEN)
        self.failIf(self.events)
                
    def testCategorizableDoesNotNotifyWhenItHasItsOwnBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.setBackgroundColor(wx.RED)
        self.registerObserver(self.categorizable.appearanceChangedEventType())
        self.category.setBackgroundColor(wx.GREEN)
        self.failIf(self.events)

    def testCategorizableDoesNotNotifyWhenItHasItsOwnFont(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.setFont(wx.SWISS_FONT)
        self.registerObserver(self.categorizable.appearanceChangedEventType())
        self.category.setFont(wx.NORMAL_FONT)
        self.failIf(self.events)

    def testCategorizableDoesNotNotifyWhenItHasItsOwnIcon(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.setIcon('icon')
        self.registerObserver(self.categorizable.appearanceChangedEventType())
        self.category.setIcon('another icon')
        self.failIf(self.events)

    def testParentForegroundColorChanged(self):
        self.registerObserver(self.appearanceChangedEventType)
        subCategory = category.Category('Subcategory')
        self.category.addChild(subCategory)
        subCategory.setParent(self.category)
        self.categorizable.addCategory(subCategory)
        subCategory.addCategorizable(self.categorizable)
        self.category.setForegroundColor(wx.RED)
        self.assertEqual(1, len(self.events))
        
    def testParentBackgroundColorChanged(self):
        self.registerObserver(self.appearanceChangedEventType)
        subCategory = category.Category('Subcategory')
        self.category.addChild(subCategory)
        subCategory.setParent(self.category)
        self.categorizable.addCategory(subCategory)
        subCategory.addCategorizable(self.categorizable)
        self.category.setBackgroundColor(wx.RED)
        self.assertEqual(1, len(self.events))

    def testParentFontChanged(self):
        self.registerObserver(self.appearanceChangedEventType)
        subCategory = category.Category('Subcategory')
        self.category.addChild(subCategory)
        subCategory.setParent(self.category)
        self.categorizable.addCategory(subCategory)
        subCategory.addCategorizable(self.categorizable)
        self.category.setFont(wx.SWISS_FONT)
        self.assertEqual(1, len(self.events))

    def testParentIconChanged(self):
        self.registerObserver(self.appearanceChangedEventType)
        subCategory = category.Category('Subcategory')
        self.category.addChild(subCategory)
        subCategory.setParent(self.category)
        self.categorizable.addCategory(subCategory)
        subCategory.addCategorizable(self.categorizable)
        self.category.setIcon('icon')
        self.assertEqual(1, len(self.events))
        
    def testAddCategoryWithForegroundColor(self):
        self.registerObserver(self.appearanceChangedEventType)
        newCategory = category.Category('New category')
        newCategory.setForegroundColor(wx.RED)
        self.categorizable.addCategory(newCategory)
        self.assertEqual(1, len(self.events))
        
    def testAddCategoryWithBackgroundColor(self):
        self.registerObserver(self.appearanceChangedEventType)
        newCategory = category.Category('New category')
        newCategory.setBackgroundColor(wx.RED)
        self.categorizable.addCategory(newCategory)
        self.assertEqual(1, len(self.events))

    def testAddCategoryWithFont(self):
        self.registerObserver(self.appearanceChangedEventType)
        newCategory = category.Category('New category')
        newCategory.setFont(wx.SWISS_FONT)
        self.categorizable.addCategory(newCategory)
        self.assertEqual(1, len(self.events))

    def testAddCategoryWithIcon(self):
        self.registerObserver(self.appearanceChangedEventType)
        newCategory = category.Category('New category')
        newCategory.setIcon('icon')
        self.categorizable.addCategory(newCategory)
        self.assertEqual(1, len(self.events))

    def testAddCategoryWithParentWithForegroundColor(self):
        self.registerObserver(self.appearanceChangedEventType)
        parentCategory = category.Category('Parent')
        parentCategory.setForegroundColor(wx.RED)
        childCategory = category.Category('Child')
        parentCategory.addChild(childCategory)
        childCategory.setParent(parentCategory)
        self.categorizable.addCategory(childCategory)
        self.assertEqual(1, len(self.events))
        
    def testAddCategoryWithParentWithBackgroundColor(self):
        self.registerObserver(self.appearanceChangedEventType)
        parentCategory = category.Category('Parent')
        parentCategory.setBackgroundColor(wx.RED)
        childCategory = category.Category('Child')
        parentCategory.addChild(childCategory)
        childCategory.setParent(parentCategory)
        self.categorizable.addCategory(childCategory)
        self.assertEqual(1, len(self.events))

    def testAddCategoryWithParentWithFont(self):
        self.registerObserver(self.appearanceChangedEventType)
        parentCategory = category.Category('Parent')
        parentCategory.setFont(wx.SWISS_FONT)
        childCategory = category.Category('Child')
        parentCategory.addChild(childCategory)
        childCategory.setParent(parentCategory)
        self.categorizable.addCategory(childCategory)
        self.assertEqual(1, len(self.events))

    def testAddCategoryWithParentWithIcon(self):
        self.registerObserver(self.appearanceChangedEventType)
        parentCategory = category.Category('Parent')
        parentCategory.setIcon('icon')
        childCategory = category.Category('Child')
        parentCategory.addChild(childCategory)
        childCategory.setParent(parentCategory)
        self.categorizable.addCategory(childCategory)
        self.assertEqual(1, len(self.events))

    def testRemoveCategoryWithForegroundColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setForegroundColor(wx.RED)
        self.registerObserver(self.appearanceChangedEventType)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(1, len(self.events))
        
    def testRemoveCategoryWithBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        self.category.setBackgroundColor(wx.RED)
        self.registerObserver(self.appearanceChangedEventType)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(1, len(self.events))

    def testRemoveCategoryWithFont(self):
        self.categorizable.addCategory(self.category)
        self.category.setFont(wx.SWISS_FONT)
        self.registerObserver(self.appearanceChangedEventType)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(1, len(self.events))

    def testRemoveCategoryWithIcon(self):
        self.categorizable.addCategory(self.category)
        self.category.setIcon('icon')
        self.registerObserver(self.appearanceChangedEventType)
        self.categorizable.removeCategory(self.category)
        self.assertEqual(1, len(self.events))

    def testForegroundColorWhenOneOutOfTwoCategoriesHasForegroundColor(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.addCategory(category.Category('Another category'))
        self.category.setForegroundColor(wx.RED)
        self.assertEqual(wx.RED, self.categorizable.foregroundColor(recursive=True))
                
    def testBackgroundColorWhenOneOutOfTwoCategoriesHasBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.addCategory(category.Category('Another category'))
        self.category.setBackgroundColor(wx.RED)
        self.assertEqual(wx.RED, self.categorizable.backgroundColor(recursive=True))

    def testFontWhenOneOutOfTwoCategoriesHasFont(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.addCategory(category.Category('Another category'))
        self.category.setFont(wx.SWISS_FONT)
        self.assertEqual(wx.SWISS_FONT, self.categorizable.font(recursive=True))

    def testIconWhenOneOutOfTwoCategoriesHasIcon(self):
        self.categorizable.addCategory(self.category)
        self.categorizable.addCategory(category.Category('Another category'))
        self.category.setIcon('icon')
        self.assertEqual('icon', self.categorizable.icon(recursive=True))

    def testForegroundColorWhenBothCategoriesHaveSameForegroundColor(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        for cat in [self.category, anotherCategory]:
            cat.setForegroundColor(wx.RED)
        self.assertEqual(wx.RED, self.categorizable.foregroundColor(recursive=True))
        
    def testBackgroundColorWhenBothCategoriesHaveSameBackgroundColor(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        for cat in [self.category, anotherCategory]:
            cat.setBackgroundColor(wx.RED)
        self.assertEqual(wx.RED, 
                         self.categorizable.backgroundColor(recursive=True))

    def testFontWhenBothCategoriesHaveSameFont(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        for cat in [self.category, anotherCategory]:
            cat.setFont(wx.SWISS_FONT)
        self.assertEqual(wx.SWISS_FONT, self.categorizable.font(recursive=True))

    def testIconWhenBothCategoriesHaveSameIcon(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        for cat in [self.category, anotherCategory]:
            cat.setIcon('icon')
        self.assertEqual('icon', self.categorizable.icon(recursive=True))

    def testForegroundColorWhenBothCategoriesHaveDifferentForegroundColors(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        self.category.setForegroundColor(wx.RED)
        anotherCategory.setForegroundColor(wx.BLUE)
        expectedColor = wx.Colour(127, 0, 127, 255)
        self.assertEqual(expectedColor, 
                         self.categorizable.foregroundColor(recursive=True))
                
    def testBackgroundColorWhenBothCategoriesHaveDifferentBackgroundColors(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        self.category.setBackgroundColor(wx.RED)
        anotherCategory.setBackgroundColor(wx.BLUE)
        expectedColor = wx.Colour(127, 0, 127, 255)
        self.assertEqual(expectedColor, 
                         self.categorizable.backgroundColor(recursive=True))

    def testFontWhenBothCategoriesHaveDifferentFontSizes(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        font = wx.SWISS_FONT
        self.category.setFont(font)
        biggerFont = wx.Font(font.GetPointSize() + 2, font.GetFamily(),
                             font.GetStyle(), font.GetWeight())
        anotherCategory.setFont(biggerFont)
        expectedFontSize = (biggerFont.GetPointSize() + font.GetPointSize()) / 2
        self.assertEqual(expectedFontSize, 
                         self.categorizable.font(recursive=True).GetPointSize())

    def testIconWhenBothCategoriesHaveDifferentIcons(self):
        self.categorizable.addCategory(self.category)
        anotherCategory = category.Category('Another category')
        self.categorizable.addCategory(anotherCategory)
        self.category.setIcon('icon')
        anotherCategory.setIcon('another_icon')
        self.failUnless(self.categorizable.icon(recursive=True) in ['icon', 'another_icon'])

    def testUseCategoryIcon(self):
        self.category.setIcon('categoryIcon')
        self.categorizable.addCategory(self.category)
        self.assertEqual('categoryIcon', self.categorizable.icon(recursive=True))

    def testDontUseCategoryIconWhenCategorizableHasItsOwnIcon(self):
        self.category.setIcon('categoryIcon')
        self.categorizable.setIcon('icon')
        self.categorizable.addCategory(self.category)
        self.assertEqual('icon', self.categorizable.icon(recursive=True))

    def testDontUseCategoryIconWhenNotRecursive(self):
        self.category.setIcon('categoryIcon')
        self.categorizable.addCategory(self.category)
        self.failIf(self.categorizable.icon(recursive=False))

    def testUseCategoryIconEvenWhenCategorizableHasARecursiveIcon(self):
        child = categorizable.CategorizableCompositeObject(subject='child')
        self.categorizable.addChild(child)
        self.categorizable.setIcon('icon')
        self.category.setIcon('categoryIcon')
        child.addCategory(self.category)
        self.assertEqual('categoryIcon', child.icon(recursive=True))

    def testUseCategorySelectedIcon(self):
        self.category.setSelectedIcon('categoryIcon')
        self.categorizable.addCategory(self.category)
        self.assertEqual('categoryIcon',
                         self.categorizable.selectedIcon(recursive=True))

    def testDontUseCategorySelectedIconWhenCategorizableHasItsOwnSelectedIcon(self):
        self.category.setSelectedIcon('categoryIcon')
        self.categorizable.setSelectedIcon('icon')
        self.categorizable.addCategory(self.category)
        self.assertEqual('icon',
                         self.categorizable.selectedIcon(recursive=True))

    def testDontUseCategorySelectedIconWhenNotRecursive(self):
        self.category.setSelectedIcon('categoryIcon')
        self.categorizable.addCategory(self.category)
        self.failIf(self.categorizable.selectedIcon(recursive=False))

    def testUseCategorySelectedIconEvenWhenCategorizableHasARecursiveSelectedIcon(self):
        child = categorizable.CategorizableCompositeObject(subject='child')
        self.categorizable.addChild(child)
        self.categorizable.setSelectedIcon('icon')
        self.category.setSelectedIcon('categoryIcon')
        child.addCategory(self.category)
        self.assertEqual('categoryIcon', child.selectedIcon(recursive=True))
    
    def testParentCategoryIncludedInChildUpwardRecursiveCategories(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.assertEqual(set([self.category]), 
                         child.categories(recursive=True, upwards=True))

    def testChildCategoryIncludedInParentDownwardRecursiveCategories(self):
        child = categorizable.CategorizableCompositeObject()
        child.addCategory(self.category)
        self.categorizable.addChild(child)
        self.assertEqual(set([self.category]), 
            self.categorizable.categories(recursive=True, upwards=False))

    def testParentCategoriesNotIncludedInNonRecursiveCategories(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.assertEqual(set(), child.categories(recursive=False))

    def testChildCategoriesNotIncludedInNonRecursiveCategories(self):
        child = categorizable.CategorizableCompositeObject()
        child.addCategory(self.category)
        self.categorizable.addChild(child)
        self.assertEqual(set(), self.categorizable.categories(recursive=False))
        
    def testGrandParentCategoryIncludedInGrandChildUpwardRecursiveCategories(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        grandchild = categorizable.CategorizableCompositeObject()
        child.addChild(grandchild)
        self.assertEqual(set([self.category]), 
                         grandchild.categories(recursive=True, upwards=True))

    def testGrandChildCategoryIncludedInGrandParentDownwardRecursiveCategories(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        grandchild = categorizable.CategorizableCompositeObject()
        child.addChild(grandchild)
        grandchild.addCategory(self.category)
        self.assertEqual(set([self.category]), 
                         self.categorizable.categories(recursive=True))
        
    def testGrandParentAndParentCategoriesIncludedInGrandChildUpwardRecursiveCategories(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        grandchild = categorizable.CategorizableCompositeObject()
        child.addChild(grandchild)
        childCategory = category.Category('Child category')
        child.addCategory(childCategory)
        self.assertEqual(set([self.category, childCategory]), 
            grandchild.categories(recursive=True, upwards=True))

    def testGrandChildAndChildCategoriesIncludedInGrandParentDownwardRecursiveCategories(self):
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        grandchild = categorizable.CategorizableCompositeObject()
        child.addChild(grandchild)
        childCategory = category.Category('Child category')
        child.addCategory(childCategory)
        grandchild.addCategory(self.category)
        self.assertEqual(set([self.category, childCategory]), 
            self.categorizable.categories(recursive=True))
        
    def testRemoveCategoryCausesChildNotification(self):
        self.categorizable.addCategory(self.category)
        child = categorizable.CategorizableCompositeObject()
        self.categorizable.addChild(child)
        self.registerObserver(self.categoryRemovedEventType, eventSource=child)
        self.categorizable.removeCategory(self.category)
        self.assertEvent(self.categoryRemovedEventType, child, self.category) 

    def testCopy(self):
        self.categorizable.addCategory(self.category)
        copy = self.categorizable.copy()
        self.assertEqual(copy.categories(), self.categorizable.categories())  # pylint: disable=E1101
        
    def testModificationEventTypes(self):  # pylint: disable=E1003
        self.assertEqual(super(categorizable.CategorizableCompositeObject,
                               self.categorizable).modificationEventTypes() + \
                         [self.categoryAddedEventType, 
                          self.categoryRemovedEventType],
                         self.categorizable.modificationEventTypes())



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import operating_system
from taskcoachlib.widgets import itemctrl
import wx.lib.mixins.listctrl


class VirtualListCtrl(itemctrl.CtrlWithItemsMixin, itemctrl.CtrlWithColumnsMixin, 
                      itemctrl.CtrlWithToolTipMixin, wx.ListCtrl):
    def __init__(self, parent, columns, selectCommand=None, editCommand=None, 
                 itemPopupMenu=None, columnPopupMenu=None, resizeableColumn=0, 
                 *args, **kwargs):
        super(VirtualListCtrl, self).__init__(parent,
            style=wx.LC_REPORT | wx.LC_VIRTUAL, columns=columns, 
            resizeableColumn=resizeableColumn, itemPopupMenu=itemPopupMenu, 
            columnPopupMenu=columnPopupMenu, *args, **kwargs)
        self.__parent = parent
        self.bindEventHandlers(selectCommand, editCommand)
            
    def bindEventHandlers(self, selectCommand, editCommand):
        # pylint: disable=W0201
        if selectCommand:
            self.selectCommand = selectCommand
            self.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.onSelect)
            self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onSelect)
            self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.onSelect)
        if editCommand:
            self.editCommand = editCommand
            self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onItemActivated)
        self.Bind(wx.EVT_SET_FOCUS, self.onSetFocus)
        
    def onSetFocus(self, event):  # pylint: disable=W0613
        # Send a child focus event to let the AuiManager know we received focus
        # so it will activate our pane
        wx.PostEvent(self, wx.ChildFocusEvent(self))
        event.Skip()
            
    def getItemWithIndex(self, rowIndex):
        return self.__parent.getItemWithIndex(rowIndex)

    def getItemText(self, domainObject, columnIndex):
        return self.__parent.getItemText(domainObject, columnIndex)
    
    def getItemTooltipData(self, domainObject):
        return self.__parent.getItemTooltipData(domainObject)
    
    def getItemImage(self, domainObject, columnIndex=0):
        return self.__parent.getItemImages(domainObject, 
                                           columnIndex)[wx.TreeItemIcon_Normal]
    
    def OnGetItemText(self, rowIndex, columnIndex):
        item = self.getItemWithIndex(rowIndex)
        return self.getItemText(item, columnIndex)

    def OnGetItemTooltipData(self, rowIndex, columnIndex):
        item = self.getItemWithIndex(rowIndex)
        return self.getItemTooltipData(item)

    def OnGetItemImage(self, rowIndex):
        item = self.getItemWithIndex(rowIndex)
        return self.getItemImage(item)
    
    def OnGetItemColumnImage(self, rowIndex, columnIndex):
        item = self.getItemWithIndex(rowIndex)
        return self.getItemImage(item, columnIndex)

    def OnGetItemAttr(self, rowIndex):
        item = self.getItemWithIndex(rowIndex)
        foreground_color = item.foregroundColor(recursive=True)
        background_color = item.backgroundColor(recursive=True)
        item_attribute_arguments = [foreground_color, background_color] 
        font = item.font(recursive=True)
        if font:
            item_attribute_arguments.append(font)
        # We need to keep a reference to the item attribute to prevent it
        # from being garbage collected too soon:
        self.__item_attribute = wx.ListItemAttr(*item_attribute_arguments)  # pylint: disable=W0142,W0201
        return self.__item_attribute
        
    def onSelect(self, event):
        event.Skip()
        self.selectCommand(event)
        
    def onItemActivated(self, event):
        ''' Override default behavior to attach the column clicked on
            to the event so we can use it elsewhere. '''
        window = self.GetMainWindow()
        if operating_system.isMac():
            window = window.GetChildren()[0]
        mouse_position = window.ScreenToClient(wx.GetMousePosition())
        index, dummy_flags, column = self.HitTest(mouse_position)
        if index >= 0:
            # Only get the column name if the hittest returned an item,
            # otherwise the item was activated from the menu or by double 
            # clicking on a portion of the tree view not containing an item.
            column = max(0, column)  # FIXME: Why can the column be -1?
            event.columnName = self._getColumn(column).name()  # pylint: disable=E1101
        self.editCommand(event)

    def RefreshAllItems(self, count):
        self.SetItemCount(count)
        if count == 0:
            self.DeleteAllItems()
        else:
            # The VirtualListCtrl makes sure only visible items are updated
            super(VirtualListCtrl, self).RefreshItems(0, count - 1)
        self.selectCommand()

    def RefreshItems(self, *items):
        ''' Refresh specific items. '''
        if len(items) <= 7:
            for item in items:
                self.RefreshItem(self.__parent.getIndexOfItem(item))
        else:
            self.RefreshAllItems(self.GetItemCount())
            
    def HitTest(self, (x, y), *args, **kwargs):
        ''' Always return a three-tuple (item, flag, column). '''
        index, flags = super(VirtualListCtrl, self).HitTest((x, y), 
                                                            *args, **kwargs)
        column = 0
        if self.InReportView():
            # Determine the column in which the user clicked
            cumulative_column_width = 0
            for column_index in range(self.GetColumnCount()):
                cumulative_column_width += self.GetColumnWidth(column_index)
                if x <= cumulative_column_width:
                    column = column_index
                    break
        return index, flags, column

    def curselection(self):
        return [self.getItemWithIndex(index) \
                for index in self.__curselection_indices()]
    
    def select(self, items):
        indices = [self.__parent.getIndexOfItem(item) for item in items]
        for index in range(self.GetItemCount()):
            self.Select(index, index in indices)
        if self.curselection():
            self.Focus(self.GetFirstSelected())        
    
    def clear_selection(self):
        ''' Unselect all selected items. '''
        for index in self.__curselection_indices():
            self.Select(index, False)

    def select_all(self):
        ''' Select all items. '''
        for index in range(self.GetItemCount()):
            self.Select(index)

    def __curselection_indices(self):
        ''' Return the indices of the currently selected items. '''
        return wx.lib.mixins.listctrl.getListCtrlSelection(self)



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
import test
from taskcoachlib import widgets


class CtrlWithColumnsTestCase(test.wxTestCase):
    def setUp(self):
        super(CtrlWithColumnsTestCase, self).setUp()
        self.column1 = widgets.Column('Column 1', 'eventType1')
        self.column2 = widgets.Column('Column 2', 'eventType2')
        self.control = self.createControl()

    def createControl(self):
        raise NotImplementedError # pragma: no cover


class CtrlWithHideableColumnsUnderTest( \
          widgets.itemctrl._CtrlWithHideableColumnsMixin, # pylint: disable=W0212 
          wx.ListCtrl): 
    pass


class CtrlWithHideableColumnsTestsMixin(object):
    def testColumnIsVisibleByDefault(self):
        self.failUnless(self.control.isColumnVisible(self.column1))
        
    def testHideColumn(self):
        self.control.showColumn(self.column1, show=False)
        self.failIf(self.control.isColumnVisible(self.column1))
        
    def testShowColumn(self):
        self.control.showColumn(self.column1, show=False)
        self.control.showColumn(self.column1, show=True)
        self.failUnless(self.control.isColumnVisible(self.column1))
        
        
class CtrlWithHideableColumnsTest(CtrlWithColumnsTestCase, 
                                  CtrlWithHideableColumnsTestsMixin):
    def createControl(self):
        return CtrlWithHideableColumnsUnderTest(self.frame, style=wx.LC_REPORT,
            columns=[self.column1, self.column2])
                
        
class CtrlWithSortableColumnsUnderTest( \
        widgets.itemctrl._CtrlWithSortableColumnsMixin, # pylint: disable=W0212 
        wx.ListCtrl):
    pass


class CtrlWithSortableColumnsTestsMixin(object):
    def assertCurrentSortColumn(self, expectedSortColumn):
        currentSortColumn = self.control._currentSortColumn() # pylint: disable=W0212
        self.assertEqual(expectedSortColumn, currentSortColumn)
        
    def testDefaultSortColumn(self):
        self.assertCurrentSortColumn(self.column1)
        
    def testShowSortColumn(self):
        self.control.showSortColumn(self.column2)
        self.assertCurrentSortColumn(self.column2)


class CtrlWithSortableColumnsTest(CtrlWithColumnsTestCase, CtrlWithSortableColumnsTestsMixin):
    def createControl(self):
        return CtrlWithSortableColumnsUnderTest(self.frame, style=wx.LC_REPORT,
            columns=[self.column1, self.column2])            
        
        
class CtrlWithColumnsUnderTest(widgets.itemctrl.CtrlWithColumnsMixin, 
                               wx.ListCtrl):
    pass


class CtrlWithColumnsTest(CtrlWithColumnsTestCase, 
        CtrlWithHideableColumnsTestsMixin, CtrlWithSortableColumnsTestsMixin):
    def createControl(self):
        # NOTE: the resizeableColumn is the column that is not hidden
        return CtrlWithColumnsUnderTest(self.frame, style=wx.LC_REPORT,
            columns=[self.column1, self.column2], resizeableColumn=1, 
            columnPopupMenu=None)


class DummyEvent(object):
    def __init__(self, eventObject):
        self.eventObject = eventObject
        
    def Skip(self, *args):
        pass
    
    def GetColumn(self):
        return 0
    
    def GetEventObject(self):
        return self.eventObject
    
    def GetPosition(self):
        return 0, 0


class ListCtrlWithColumnPopupMenuTest(CtrlWithColumnsTestCase):
    def createControl(self):
        return CtrlWithColumnsUnderTest(self.frame, style=wx.LC_REPORT, 
            columns=[self.column1, self.column2], resizeableColumn=1, 
            columnPopupMenu=wx.Menu())

    @test.skipOnPlatform('__WXGTK__') # Popup menu hangs the test
    def testColumnHeaderPopupMenu(self):
        self.control.onColumnPopupMenu(DummyEvent(self.control))


class HyperListTreeCtrlWithColumnPopupMenuTest(CtrlWithColumnsTestCase):
    def createControl(self):
        return widgets.TreeListCtrl(self.frame, [self.column1, self.column2], 
            None, None, None, None, columnPopupMenu=wx.Menu())

    @test.skipOnPlatform('__WXGTK__') # Popup menu hangs the test
    def testColumnHeaderPopupMenu(self):
        self.control.onColumnPopupMenu(DummyEvent(self.control))



# -*- coding: utf-8 -*-

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
import test
from taskcoachlib import gui, config
from taskcoachlib.gui import uicommand
from taskcoachlib.domain import task, category, date
from taskcoachlib.thirdparty.pubsub import pub


class MockViewerContainer(object):
    def __init__(self):
        self.__sortBy = 'subject'
        self.__ascending = True
        self.selection = []
        self.showingCategories = False
    
    def settingsSection(self):
        return 'section'
    
    def curselection(self):
        return self.selection  # pragma: no cover
    
    def isShowingCategories(self):
        return self.showingCategories  # pragma: no cover
    
    def isSortable(self):
        return True
        
    def sortBy(self, sortKey):
        self.__sortBy = sortKey
        
    def isSortedBy(self, sortKey):
        return sortKey == self.__sortBy

    def isSortOrderAscending(self, *args, **kwargs):  # pylint: disable=W0613
        return self.__ascending
    
    def setSortOrderAscending(self, ascending=True):
        self.__ascending = ascending

    def isSortByTaskStatusFirst(self):
        return True
    
    def isSortCaseSensitive(self):
        return True

    def getSortUICommands(self):
        return [uicommand.ViewerSortOrderCommand(viewer=self), 
                uicommand.ViewerSortCaseSensitive(viewer=self), 
                uicommand.ViewerSortByTaskStatusFirst(viewer=self), 
                None, 
                uicommand.ViewerSortByCommand(viewer=self, value='subject',
                    menuText='Sub&ject', helpText='help'), 
                uicommand.ViewerSortByCommand(viewer=self, value='description',
                    menuText='&Description', helpText='help')]
        

class MenuTestCase(test.wxTestCase):
    def setUp(self):
        super(MenuTestCase, self).setUp()
        self.frame.viewer = MockViewerContainer()
        self.menu = gui.menu.Menu(self.frame)
        menuBar = wx.MenuBar()
        menuBar.Append(self.menu, 'menu')
        self.frame.SetMenuBar(menuBar)


class MenuTest(MenuTestCase):
    def testLenEmptyMenu(self):
        self.assertEqual(0, len(self.menu))

    def testLenNonEmptyMenu(self):
        self.menu.AppendSeparator()
        self.assertEqual(1, len(self.menu))


class MenuWithBooleanMenuItemsTestCase(MenuTestCase):
    def setUp(self):
        super(MenuWithBooleanMenuItemsTestCase, self).setUp()
        self.settings = config.Settings(load=False)
        self.commands = self.createCommands()

    def createCommands(self):
        raise NotImplementedError  # pragma: no cover
    
    def assertMenuItemsChecked(self, *expectedStates):
        for command in self.commands:
            self.menu.appendUICommand(command)
        self.menu.openMenu()
        for index, shouldBeChecked in enumerate(expectedStates):
            isChecked = self.menu.FindItemByPosition(index).IsChecked()
            if shouldBeChecked:
                self.failUnless(isChecked)
            else:
                self.failIf(isChecked)


class MenuWithCheckItemsTest(MenuWithBooleanMenuItemsTestCase):
    def createCommands(self):
        return [uicommand.UICheckCommand(settings=self.settings,
                                         section='view', setting='statusbar')]

    def testCheckedItem(self):
        self.settings.set('view', 'statusbar', 'True')
        self.assertMenuItemsChecked(True)

    def testUncheckedItem(self):
        self.settings.set('view', 'statusbar', 'False')
        self.assertMenuItemsChecked(False)


class MenuWithRadioItemsTest(MenuWithBooleanMenuItemsTestCase):
    def createCommands(self):
        return [uicommand.UIRadioCommand(settings=self.settings, 
                                         section='view', setting='toolbar', 
                                         value=value) \
                for value in [None, (16, 16)]]

    def testRadioItem_FirstChecked(self):
        self.settings.setvalue('view', 'toolbar', None)
        self.assertMenuItemsChecked(True, False)

    def testRadioItem_SecondChecked(self):
        self.settings.setvalue('view', 'toolbar', (16, 16))
        self.assertMenuItemsChecked(False, True)


class MockIOController:
    def __init__(self):
        self.openCalled = False
        
    def open(self, *args, **kwargs):  # pylint: disable=W0613
        self.openCalled = True


class RecentFilesMenuTest(test.wxTestCase):
    def setUp(self):
        super(RecentFilesMenuTest, self).setUp()
        self.ioController = MockIOController()
        self.settings = config.Settings(load=False)
        self.initialFileMenuLength = len(self.createFileMenu())
        self.filename1 = 'c:/Program Files/TaskCoach/test.tsk'
        self.filename2 = 'c:/two.tsk'
        self.filenames = []
        
    def createFileMenu(self):
        return gui.menu.FileMenu(self.frame, self.settings, 
                                 self.ioController, None)
        
    def setRecentFilesAndCreateMenu(self, *filenames):
        self.addRecentFiles(*filenames)
        self.menu = self.createFileMenu()  # pylint: disable=W0201
    
    def addRecentFiles(self, *filenames):
        self.filenames.extend(filenames)
        self.settings.set('file', 'recentfiles', str(list(self.filenames)))
        
    def assertRecentFileMenuItems(self, *expectedFilenames):
        expectedFilenames = expectedFilenames or self.filenames
        self.openMenu()
        numberOfMenuItemsAdded = len(expectedFilenames)
        if numberOfMenuItemsAdded > 0:
            numberOfMenuItemsAdded += 1  # the extra separator
        self.assertEqual(self.initialFileMenuLength + numberOfMenuItemsAdded, len(self.menu))
        for index, expectedFilename in enumerate(expectedFilenames):
            menuItem = self.menu.FindItemByPosition(self.initialFileMenuLength-1 + index)
            # Apparently the '&' can also be a '_' (seen on Ubuntu)
            expectedLabel = u'&%d %s' % (index + 1, expectedFilename)
            self.assertEqual(expectedLabel[1:], menuItem.GetText()[1:])
    
    def openMenu(self):
        self.menu.onOpenMenu(wx.MenuEvent(menu=self.menu))
        
    def testNoRecentFiles(self):
        self.setRecentFilesAndCreateMenu()
        self.assertRecentFileMenuItems()
        
    def testOneRecentFileWhenCreatingMenu(self):
        self.setRecentFilesAndCreateMenu(self.filename1)
        self.assertRecentFileMenuItems()
        
    def testTwoRecentFilesWhenCreatingMenu(self):
        self.setRecentFilesAndCreateMenu(self.filename1, self.filename2)
        self.assertRecentFileMenuItems()
                
    def testAddRecentFileAfterCreatingMenu(self):
        self.setRecentFilesAndCreateMenu()
        self.addRecentFiles(self.filename1)
        self.assertRecentFileMenuItems()
        
    def testOneRecentFileWhenCreatingMenuAndAddOneRecentFileAfterCreatingMenu(self):
        self.setRecentFilesAndCreateMenu(self.filename1)
        self.addRecentFiles(self.filename2)
        self.assertRecentFileMenuItems()
        
    def testOpenARecentFile(self):
        self.setRecentFilesAndCreateMenu(self.filename1)
        self.openMenu()
        menuItem = self.menu.FindItemByPosition(self.initialFileMenuLength - 1)
        self.menu.invokeMenuItem(menuItem)
        self.failUnless(self.ioController.openCalled)
        
    def testNeverShowMoreThanTheMaximumNumberAllowed(self):
        self.setRecentFilesAndCreateMenu(self.filename1, self.filename2)
        self.settings.set('file', 'maxrecentfiles', '1')
        self.assertRecentFileMenuItems(self.filename1)


class ViewMenuTestCase(test.wxTestCase):
    def setUp(self):
        super(ViewMenuTestCase, self).setUp()
        self.settings = config.Settings(load=False)
        self.viewerContainer = MockViewerContainer()
        self.menuBar = wx.MenuBar()
        self.parentMenu = wx.Menu()
        self.menuBar.Append(self.parentMenu, 'parentMenu')
        self.menu = self.createMenu()
        self.parentMenu.AppendSubMenu(self.menu, 'menu')
        self.frame.SetMenuBar(self.menuBar)
        
    def createMenu(self):
        self.frame.viewer = self.viewerContainer
        menu = gui.menu.SortMenu(self.frame, self.parentMenu, 'menu')
        menu.updateMenu()
        return menu
        
    def testSortOrderAscending(self):
        self.viewerContainer.setSortOrderAscending(True)
        self.menu.UpdateUI()
        self.menu.openMenu()
        self.failUnless(self.menu.FindItemByPosition(0).IsChecked())
        
    def testSortOrderDescending(self):
        self.viewerContainer.setSortOrderAscending(False)
        self.menu.UpdateUI()
        self.menu.openMenu()
        self.failIf(self.menu.FindItemByPosition(0).IsChecked())

    def testSortBySubject(self):
        self.viewerContainer.sortBy('subject')
        self.menu.UpdateUI()
        self.menu.openMenu()
        self.failUnless(self.menu.FindItemByPosition(4).IsChecked())
        self.failIf(self.menu.FindItemByPosition(5).IsChecked())

    def testSortByDescription(self):
        self.viewerContainer.sortBy('description')
        self.menu.UpdateUI()
        self.menu.openMenu()
        self.failIf(self.menu.FindItemByPosition(4).IsChecked())
        self.failUnless(self.menu.FindItemByPosition(5).IsChecked())


class StartEffortForTaskMenuTest(test.wxTestCase):
    def setUp(self):
        task.Task.settings = config.Settings(load=False)
        self.tasks = task.TaskList()
        self.menu = gui.menu.StartEffortForTaskMenu(self.frame, self.tasks)
        
    def addTask(self, subject='Subject'):
        newTask = task.Task(subject=subject, plannedStartDateTime=date.Now())
        self.tasks.append(newTask)
        return newTask
    
    def addParentAndChild(self, parentSubject='Subject', 
                          childSubject='Subject'):
        parent = self.addTask(parentSubject)
        child = self.addTask(childSubject)
        parent.addChild(child)
        return parent, child
    
    def testMenuIsEmptyInitially(self):
        self.assertEqual(0, len(self.menu))

    def testNewTasksAreAdded(self):
        self.addTask()
        self.assertEqual(1, len(self.menu))

    def testDeletedTasksAreRemoved(self):
        newTask = self.addTask()
        self.tasks.remove(newTask)
        self.assertEqual(0, len(self.menu))
    
    def testNewChildTasksAreAdded(self):
        self.addParentAndChild()
        self.assertEqual(2, len(self.menu))
        
    def testDeletedChildTasksAreRemoved(self):
        child = self.addParentAndChild()[1]
        self.tasks.remove(child)
        self.assertEqual(1, len(self.menu))
        
    def testTaskWithNonAsciiSubject(self):
        self.addParentAndChild(u'Jérôme', u'Jîrôme')
        self.menu.updateMenuItems()
        self.assertEqual(2, len(self.menu))


class ToggleCategoryMenuTest(test.wxTestCase):
    def setUp(self):
        self.categories = category.CategoryList()
        self.category1 = category.Category('Category 1')
        self.category2 = category.Category('Category 2')
        self.viewerContainer = MockViewerContainer() 
        self.menu = gui.menu.ToggleCategoryMenu(self.frame, self.categories,
                                                self.viewerContainer)
        
    def setUpSubcategories(self):
        self.category1.addChild(self.category2)
        self.categories.append(self.category1)
        
    def testMenuInitiallyEmpty(self):
        self.assertEqual(0, len(self.menu))
        
    def testOneCategory(self):
        self.categories.append(self.category1)
        self.assertEqual(1, len(self.menu))

    def testTwoCategories(self):
        self.categories.extend([self.category1, self.category2])
        self.assertEqual(2, len(self.menu))
        
    def testSubcategory(self):
        self.setUpSubcategories()
        self.assertEqual(3, len(self.menu))

    def testSubcategorySubmenuLabel(self):
        self.setUpSubcategories()
        self.assertEqual(gui.menu.ToggleCategoryMenu.subMenuLabel(self.category1), 
                         self.menu.GetMenuItems()[2].GetLabel())

    def testSubcategorySubmenuItemLabel(self):
        self.setUpSubcategories()
        subMenu = self.menu.GetMenuItems()[2].GetSubMenu()
        label = subMenu.GetMenuItems()[0].GetLabel()
        self.assertEqual(self.category2.subject(), label)
                
    def testMutualExclusiveSubcategories_AreCheckItems(self):
        self.category1.makeSubcategoriesExclusive()
        self.setUpSubcategories()
        category3 = category.Category('Category 3')
        self.category1.addChild(category3)
        subMenu = self.menu.GetMenuItems()[2].GetSubMenu()
        for subMenuItem in subMenu.GetMenuItems():
            self.assertEqual(wx.ITEM_CHECK, subMenuItem.GetKind())
        
    def testMutualExclusiveSubcategories_NoneChecked(self):
        self.category1.makeSubcategoriesExclusive()
        self.setUpSubcategories()
        category3 = category.Category('Category 3')
        self.category1.addChild(category3)
        subMenuItems = self.menu.GetMenuItems()[2].GetSubMenu().GetMenuItems()
        checkedItems = [item for item in subMenuItems if item.IsChecked()]
        self.failIf(checkedItems)
        
    def testMutualExclusiveSubcategoriesWithSubcategories(self):
        self.category1.makeSubcategoriesExclusive()
        self.setUpSubcategories()
        category3 = category.Category('Category 3')
        self.category1.addChild(category3)
        category4 = category.Category('Category 4')
        category3.addChild(category4)
        subMenuItems = self.menu.GetMenuItems()[2].GetSubMenu().GetMenuItems()
        checkedItems = [item for item in subMenuItems if item.IsChecked()]
        self.failIf(checkedItems)


class TaskTemplateMenuTest(test.wxTestCase):
    def testMenuIsUpdatedWhenTemplatesAreSaved(self):
        uicommands = [None]  # Just a separator for testing purposes
        
        class TaskTemplateMenu(gui.menu.TaskTemplateMenu):
            def getUICommands(self):
                return uicommands
            
        settings = config.Settings(load=False)
        taskList = task.TaskList()
        menu = TaskTemplateMenu(self.frame, taskList, settings)
        self.assertEqual(1, len(menu))
        uicommands.append(None)  # Add another separator
        pub.sendMessage('templates.saved')
        self.assertEqual(2, len(menu))
        


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import wx


def timeFunction(func):
	"""Decorator which displays the total time a function used at exit, grouped by stack frame."""
	import atexit, time, traceback
	func.elapsed = dict()
	def wrapper(*args, **kwargs):
		t0 = time.time()
		try:
			return func(*args, **kwargs)
		finally:
			exc = '\n'.join(traceback.format_stack(limit=2)[:-1])
			elapsed, count = func.elapsed.get(exc, (0.0, 0))
			func.elapsed[exc] = (elapsed + (time.time() - t0), count + 1)

	def printit():
		elapsed = [(tm, count, exc) for exc, (tm, count) in func.elapsed.items()]
		elapsed.sort()
		totalElapsed = 0.0
		totalCount = 0
		for tm, count, exc in elapsed:
			print '========== %d ms / %d calls' % (int(tm * 1000), count)
			print exc
			totalElapsed += tm
			totalCount += count
		print '==========='
		print 'Total time: %d ms' % int(totalElapsed * 1000)
		print 'Total calls: %d' % totalCount

	atexit.register(printit)

	return wrapper

def copyDate(value):
	""" Simple method for copy the date (Y,M,D).
	"""
	return wx.DateTimeFromDMY(value.GetDay(), value.GetMonth(), value.GetYear())

def copyDateTime(value):
	""" Return a copy of input wxDateTime object
	"""
	if value.IsValid():
		return wx.DateTimeFromDMY(
			value.GetDay(), 
			value.GetMonth(),
			value.GetYear(),
			value.GetHour(),
			value.GetMinute(),
			value.GetSecond(),
			value.GetMillisecond(),
		)
	else:
		return wx.DateTime()

def setToWeekDayInSameWeek(day, offset, startDay=1):
	"""wxDateTime's    SetToWeekDayInSameWeek   appears    to   be
	buggish. When told that the  week starts on Monday, it results
	in   the  following   'week'  on   Jan,  31st,   2010:  31/01,
	25/01-30/01..."""
	# Loop backwards until we find the start day
	while True:
		if day.GetWeekDay() == startDay:
			break
		day.SubtractDS(wx.DateSpan(days=1))
	day.AddDS(wx.DateSpan(days=offset))
	return day
